/* Breadcrumbs.java

	Purpose:
		
	Description:
		
	History:
		11:18 AM 2022/9/8, Created by jumperchen

Copyright (C) 2022 Potix Corporation. All Rights Reserved.
*/
package org.zkoss.chart;

import java.util.LinkedHashMap;
import java.util.Map;

import org.zkoss.chart.util.DynamicalAttribute;
import org.zkoss.chart.util.MapsHelper;
import org.zkoss.json.JavaScriptValue;
import org.zkoss.lang.Generics;

/**
 * Options for the breadcrumbs, the navigation at the top leading the way up through
 * the traversed levels.
 * <p>
 * All the options in this class support {@link DynamicalAttribute}.
 * @author jumperchen
 * @since 10.2.1.0
 */
public class Breadcrumbs extends Optionable {
	private enum Attrs implements PlotAttribute, DynamicalAttribute {
		buttonSpacing,
		buttonTheme,
		// TODO
		events,
		floating,
		format,
		formatter,
		position,
		relativeTo,
		rtl,
		separator,
		showFullPath,
		style,
		useHTML,
		zIndex
	}

	/**
	 * Returns the default padding for each button and separator in each direction.
	 * <p>Default: 5
	 */
	public Number getButtonSpacing() {
		return getAttr(Attrs.buttonSpacing, 5).asNumber();
	}

	/**
	 * Sets the default padding for each button and separator in each direction.
	 * <p>Default: 5
	 */
	public void setButtonSpacing(Number buttonSpacing) {
		setAttr(Attrs.buttonSpacing, buttonSpacing);
	}


	/**
	 * Returns the collection of attributes for the buttons.
	 * The object takes SVG attributes like fill, stroke, stroke-width, as well as style,
	 * a collection of CSS properties for the text.
	 * <p>Default: null
	 */
	public <K, V> Map<K, V> getButtonTheme() {
		return Generics.cast(getAttr(Attrs.buttonTheme));
	}

	/**
	 * Sets the collection of attributes for the buttons.
	 * The object takes SVG attributes like fill, stroke, stroke-width, as well as style,
	 * a collection of CSS properties for the text.
	 */
	public void setButtonTheme(String buttonTheme) {
		setButtonTheme(
				MapsHelper.parse(new LinkedHashMap<>(), buttonTheme, ':', ';', '\''));
	}

	/**
	 * Sets the collection of attributes for the buttons.
	 * The object takes SVG attributes like fill, stroke, stroke-width, as well as style,
	 * a collection of CSS properties for the text.
	 */
	public <K, V> void setButtonTheme(Map<K, V> buttonTheme) {
		setAttr(Attrs.buttonTheme, buttonTheme);
	}

	/**
	 * When the breadcrumbs are floating, the plot area will not move to make space
	 * for it. By default, the chart will not make space for the buttons.
	 * This property won't work when positioned in the middle.
	 *
	 * <p>Defaults to false.
	 */
	public boolean isFloating() {
		return getAttr(Attrs.floating, false).asBoolean();
	}

	/**
	 * When the breadcrumbs are floating, the plot area will not move to make space
	 * for it. By default, the chart will not make space for the buttons.
	 * This property won't work when positioned in the middle.
	 *
	 * <p>Defaults to false.
	 */
	public void setFloating(boolean floating) {
		setAttr(Attrs.floating, floating);
	}

	/**
	 * Returns a format string for the breadcrumbs button.
	 * Variables are enclosed by curly brackets.
	 * Available values are passed in the declared point options.
	 * <p>
	 * Default: null
	 */
	public String getFormat() {
		return getAttr(Attrs.format, null).asString();
	}

	/**
	 * Sets a format string for the breadcrumbs button.
	 * Variables are enclosed by curly brackets.
	 * Available values are passed in the declared point options.
	 */
	public void setFormat(String format) {
		setAttr(Attrs.format, format);
	}

	/**
	 * Returns the callback JavaScript function to format the breadcrumb text from scratch.
	 * <p>Default: null
	 */
	public JavaScriptValue getFormatter() {
		return (JavaScriptValue) getAttr(Attrs.formatter, null).asValue();
	}

	/**
	 * Sets the callback JavaScript function to format the breadcrumb text from scratch.
	 * <p>Default: null
	 */
	public void setFormatter(JavaScriptValue formatter) {
		setAttr(Attrs.formatter, formatter);
	}

	/**
	 * Returns the position for the button row.
	 * The breadcrumbs buttons will be aligned properly for the default chart layout
	 * (title, subtitle, legend, range selector) for the custom chart layout set
	 * the position properties.
	 * <p>Default: null
	 */
	public Position getPosition() {
		return (Position) getAttr(Attrs.position);
	}

	/**
	 * Sets the position for the button row.
	 * The breadcrumbs buttons will be aligned properly for the default chart layout
	 * (title, subtitle, legend, range selector) for the custom chart layout set
	 * the position properties.
	 * <p>Default: null
	 */
	public void setPosition(Position position) {
		setAttr(Attrs.position, position);
	}

	/**
	 * What box to align the button to. Can be either plotBox or spacingBox.
	 *
	 * <p>Defaults to {@code "plotBox"}.
	 */
	public String getRelativeTo() {
		return getAttr(Attrs.relativeTo, "plotBox").asString();
	}

	/**
	 * What box to align the button to. Can be either plotBox or spacingBox.
	 *
	 * <p>Defaults to {@code "plotBox"}.
	 */
	public void setRelativeTo(String relativeTo) {
		setAttr(Attrs.relativeTo, relativeTo);
	}
	/**
	 * Returns whether to reverse the order of buttons. This is common in Arabic and Hebrew.
	 * <p>Defaults to false.
	 */
	public boolean isRtl() {
		return getAttr(Attrs.rtl, false).asBoolean();
	}

	/**
	 * Returns whether to reverse the order of buttons. This is common in Arabic and Hebrew.
	 *
	 * <p>Defaults to false.
	 */
	public void setRtl(boolean rtl) {
		setAttr(Attrs.rtl, rtl);
	}

	/**
	 * Returns the options object for Breadcrumbs separator.
	 * <p>Default: null
	 */
	public Separator getSeparator() {
		return (Separator) getAttr(Attrs.separator, null).asValue();
	}

	/**
	 * Sets the options object for Breadcrumbs separator.
	 * <p>Default: null
	 */
	public void setSeparator(Separator separator) {
		setAttr(Attrs.separator, separator);
	}

	/**
	 * Show full path or only a single button.
	 * <p>Default: {@code true}
	 */
	public boolean isShowFullPath() {
		return getAttr(Attrs.showFullPath, false).asBoolean();
	}

	/**
	 * Show full path or only a single button.
	 * <p>Default: {@code true}
	 */
	public void setShowFullPath(boolean showFullPath) {
		setAttr(Attrs.showFullPath, showFullPath);
	}
	/**
	 * Returns CSS styles for all breadcrumbs.
	 * <p>Default: null
	 */
	public <K, V> Map<K, V> getStyle() {
		return Generics.cast(getAttr(Attrs.style));
	}

	/**
	 * Sets CSS styles for all breadcrumbs.
	 * <p>Default: null
	 */
	public void setStyle(String style) {
		setStyle(MapsHelper.parse(new LinkedHashMap<>(), style, ':', ';', '\''));
	}

	/**
	 * Sets CSS styles for all breadcrumbs.
	 * <p>Default: null
	 */
	public <K, V> void setStyle(Map<K, V> style) {
		setAttr(Attrs.style, style);
	}
	/**
	 * Returns whether to use HTML to render the breadcrumbs items texts.
	 * <p>
	 * Default: false.
	 */
	public boolean isUseHTML() {
		return getAttr(Attrs.useHTML, false).asBoolean();
	}

	/**
	 * Sets whether to use HTML to render the breadcrumbs items texts.
	 * <p>
	 * Default: false.
	 */
	public void setUseHTML(boolean useHTML) {
		setAttr(Attrs.useHTML, useHTML);
	}

	/**
	 * Returns the z index of the breadcrumbs group.
	 * <p>Default: {@code 7}
	 */
	public Number getZIndex() {
		return getAttr(Attrs.zIndex, 7).asNumber();
	}

	/**
	 * Sets the z index of the breadcrumbs group.
	 * <p>Default: {@code 7}
	 */
	public void setZIndex(Number zIndex) {
		setAttr(Attrs.zIndex, zIndex);
	}
}
