/* Caption.java

	Purpose:
		
	Description:
		
	History:
		Thu Nov 14 17:39:16 CST 2019, Created by rudyhuang

Copyright (C) 2019 Potix Corporation. All Rights Reserved.
*/
package org.zkoss.chart;

import java.util.LinkedHashMap;
import java.util.Map;

import org.zkoss.chart.util.DynamicalAttribute;
import org.zkoss.chart.util.MapsHelper;
import org.zkoss.lang.Generics;

/**
 * The chart's caption, which will render below the chart and will be part of exported charts.
 * <p>
 * All the options in this class support {@link DynamicalAttribute}.
 *
 * @author rudyhuang
 * @since 7.2.1.0
 */
public class Caption extends Optionable {
	private enum Attrs implements PlotAttribute, DynamicalAttribute {
		align, floating, margin, style, text, useHTML, verticalAlign, x, y
	}

	/**
	 * Returns the horizontal alignment of the caption.
	 * Can be one of "left", "center" and "right".
	 * <p>
	 * Default: "left".
	 */
	public String getAlign() {
		return getAttr(Attrs.align, "left").asString();
	}

	/**
	 * Sets the horizontal alignment of the caption.
	 * Can be one of "left", "center" and "right".
	 */
	public void setAlign(String align) {
		if (!("left".equals(align) || "center".equals(align) || "right".equals(align)))
			throw new IllegalArgumentException("Unsupported align: [" + align + "]");
		setAttr(Attrs.align, align, "left");
	}

	/**
	 * Returns when the caption is floating, the plot area will not move to make space for it.
	 * <p>
	 * Default: false.
	 */
	public boolean isFloating() {
		return getAttr(Attrs.floating, false).asBoolean();
	}

	/**
	 * Sets when the caption is floating, the plot area will not move to make space for it.
	 */
	public void setFloating(boolean floating) {
		setAttr(Attrs.floating, floating);
	}

	/**
	 * Returns CSS styles for the caption.
	 * <p>
	 * Defaults to:
	 *
	 * <pre>
	 * color: #666666;
	 * </pre>
	 * @param <K> type for the key
	 * <p>
	 * if style was set using {@link #setStyle(String)}, type will be String
	 * <p>
	 * if style was set using {@link #setStyle(Map)}, type will be the same as the type from the key of the map
	 * <p>
	 * if style was not set, default style will be applied, and type will be String
	 * @param <V> type for the value
	 * <p>
	 * if style was set using {@link #setStyle(String)}, type will be String
	 * <p>
	 * if style was set using {@link #setStyle(Map)}, type will be the same as the type from the key of the map
	 * <p>
	 * if style was not set, default style will be applied, and type will be String
	 */
	public <K, V> Map<K, V> getStyle() {
		if (!containsKey(Subtitle.Attrs.style)) {
			setStyle("color: #666666;");
		}
		return Generics.cast(getAttr(Attrs.style));
	}

	/**
	 * Sets CSS styles for the caption.
	 * <p>
	 * This method allows <b>simple</b> settings like
	 * <pre>{@code
	 * label.setStyle("fontWeight: 'bold'; color: 'white'; textShadow: '0px 1px 2px black'");
	 * }</pre>
	 * <p>
	 * In the format of CSS key:value, and ";" separating multiple key:value pairs,
	 * if you have more a complex CSS styling, we recommend using {@link #setStyle(Map)}
	 * @param style one or many CSS key:value pairs in a single line string format
	 */
	public void setStyle(String style) {
		setStyle(MapsHelper.parse(new LinkedHashMap<String, String>(), style, ':', ';', '\''));
	}

	/**
	 * Sets CSS styles for the caption.
	 * <p>
	 * This method allows <b>complex</b> styling like:
	 * <pre>
	 * {@code
	 * Map<String, Object> styles = new HashMap<String, Object>();
	 * styles.put("fill", "#E8E8EA");
	 * styles.put("stroke", "#6E6E78");
	 * styles.put("stroke-width", "1");
	 * styles.put("r", "0");
	 * Map<String, Object> states = new HashMap<String, Object>();
	 * Map<String, String> hover = new HashMap<String, String>();
	 * hover.put("fill", "#F4F4F6");
	 * hover.put("stroke", "#6E6E78");
	 * hover.put("stroke-width", "1");
	 * hover.put("r", "0");
	 * states.put("hover", hover);
	 * styles.put("states", states);
	 * Map<String, String> style = new HashMap<String, String>();
	 * style.put("color", "#4D5663");
	 * style.put("fontSize", "12px");
	 * style.put("fontFamily", "'Arial'");
	 * style.put("fontWeight", "normal");
	 * style.put("padding", "10px");
	 * styles.put("style", style);
	 * }</pre>
	 * which will generate the final results like:
	 * <pre>{@code
	 * {
	 *     "fill": "#E8E8EA",
	 *     "stroke": "#6E6E78",
	 *     "stroke-width": "1",
	 *     "r": "0",
	 *     "states":
	 *     {
	 *         "hover":
	 *         {
	 *             "fill": "#F4F4F6",
	 *             "stroke": "#6E6E78",
	 *             "stroke-width": "1",
	 *             "r": "0"
	 *         }
	 *     },
	 *     "style":
	 *     {
	 *         "color": "#4D5663",
	 *         "fontSize": "12px",
	 *         "fontFamily": "'Arial'",
	 *         "fontWeight": "normal",
	 *         "padding": "10px"
	 *     }
	 * }
	 * }</pre>
	 * @param <K> Type for the key, usually String
	 * @param <V> Type for the value, usually String. For nested maps, use Object or any other type.
	 * @param style the style object
	 */
	public <K, V> void setStyle(Map<K, V> style) {
		setAttr(Attrs.style, style, NOT_NULL_VALUE);
	}

	/**
	 * Returns the caption text of the chart.
	 * <p>
	 * Default: "".
	 */
	public String getText() {
		return getAttr(Attrs.text, "").asString();
	}

	/**
	 * Sets the caption text of the chart.
	 */
	public void setText(String text) {
		setAttr(Attrs.text, text);
	}

	/**
	 * Returns whether to <a href="https://www.highcharts.com/docs/chart-concepts/labels-and-string-formatting#html">use
	 * HTML</a> to render the text.
	 * <p>
	 * Default: false.
	 */
	public boolean isUseHTML() {
		return getAttr(Attrs.useHTML, false).asBoolean();
	}

	/**
	 * Sets whether to <a href="https://www.highcharts.com/docs/chart-concepts/labels-and-string-formatting#html">use
	 * HTML</a> to render the text.
	 */
	public void setUseHTML(boolean useHTML) {
		setAttr(Attrs.useHTML, useHTML, false);
	}

	/**
	 * Returns the vertical alignment of the caption. Can be one of "top",
	 * "middle" and "bottom". When middle, the caption behaves as floating.
	 * <p>
	 * Default: bottom.
	 */
	public String getVerticalAlign() {
		return getAttr(Attrs.verticalAlign, "bottom").asString();
	}

	/**
	 * Sets the vertical alignment of the caption. Can be one of "top",
	 * "middle" and "bottom". When middle, the caption behaves as floating.
	 */
	public void setVerticalAlign(String verticalAlign) {
		if (!("top".equals(verticalAlign) || "middle".equals(verticalAlign) || "bottom".equals(verticalAlign)))
			throw new IllegalArgumentException("Unsupported vertical align: [" + verticalAlign + "]");
		setAttr(Attrs.verticalAlign, verticalAlign, "bottom");
	}

	/**
	 * Returns the x position of the caption relative to the alignment within
	 * {@link Chart#setSpacingLeft(Number)} and
	 * {@link Chart#setSpacingRight(Number)}.
	 * <p>
	 * Default: 0.
	 */
	public Number getX() {
		return getAttr(Attrs.x, 0).asNumber();
	}

	/**
	 * Sets the x position of the caption relative to the alignment within
	 * {@link Chart#setSpacingLeft(Number)} and
	 * {@link Chart#setSpacingRight(Number)}.
	 */
	public void setX(Number x) {
		setAttr(Attrs.x, x, 0);
	}

	/**
	 * Returns the y position of the caption relative to the alignment within
	 * {@link Chart#setSpacingTop(Number)} and
	 * {@link Chart#setSpacingBottom(Number)}.
	 * <p>
	 * Default: null.
	 */
	public Number getY() {
		return getAttr(Attrs.y, null).asNumber();
	}

	/**
	 * Sets the y position of the caption relative to the alignment within
	 * {@link Chart#setSpacingTop(Number)} and
	 * {@link Chart#setSpacingBottom(Number)}.
	 */
	public void setY(Number y) {
		setAttr(Attrs.y, y);
	}
}
