/** Chart.java.

	Purpose:
		
	Description:
		
	History:
		11:52:02 AM Jan 13, 2014, Created by RaymondChao

Copyright (C) 2014 Potix Corporation. All Rights Reserved.
 */

package org.zkoss.chart;

import java.util.LinkedHashMap;
import java.util.Map;

import org.zkoss.chart.options3D.Options3D;
import org.zkoss.chart.plotOptions.SeriesPlotOptions;
import org.zkoss.chart.util.DynamicalAttribute;
import org.zkoss.chart.util.MapsHelper;
import org.zkoss.json.JavaScriptValue;
import org.zkoss.lang.Generics;

/**
 * An Chart options regarding the chart area and plot area as well as general
 * chart options.
 * 
 * <p> All the options in this class support {@link DynamicalAttribute}.
 * 
 * @author jumperchen
 * @author RaymondChao
 */
public class Chart extends Optionable {
	private enum Attrs implements PlotAttribute, DynamicalAttribute {
		alignTicks,
		animation,
		backgroundColor,
		borderColor,
		borderRadius,
		borderWidth,
		className,
		// defaultSeriesType, Same as type
		events, //Same as onXXX unless you want to run JS on client side
		height,
		ignoreHiddenSeries,
		inverted,
		margin,
		marginBottom,
		marginLeft,
		marginRight,
		marginTop,
		options3d,
		panKey,
		panning,
		parallelAxes,
		parallelCoordinates,
		pinchType,
		plotBackgroundColor,
		plotBackgroundImage,
		plotBorderColor,
		plotBorderWidth,
		plotShadow,
		polar,
		reflow,
		// renderTo,
		resetZoomButton,
		scrollablePlotArea,
		selectionMarkerFill,
		shadow,
		showAxes,
		spacing,
		spacingBottom,
		spacingLeft,
		spacingRight,
		spacingTop,
		style,
		styledMode,
		type,
		width,
		zoomType,
		enableZoomSelection
	}

	/**
	 * Returns when using multiple axis, the ticks of two or more opposite axes
	 * will automatically be aligned by adding ticks to the axis or axes with
	 * the least ticks. This can be prevented by setting <code>alignTicks</code>
	 * to false. If the grid lines look messy, it's a good idea to hide them for
	 * the secondary axis by setting <code>gridLineWidth</code> to 0.
	 * <p>
	 * Default: true.
	 */
	public boolean isAlignTicks() {
		return getAttr(Attrs.alignTicks, true).asBoolean();
	}

	/**
	 * Sets when using multiple axis, the ticks of two or more opposite axes
	 * will automatically be aligned by adding ticks to the axis or axes with
	 * the least ticks. This can be prevented by setting <code>alignTicks</code>
	 * to false. If the grid lines look messy, it's a good idea to hide them for
	 * the secondary axis by setting <code>gridLineWidth</code> to 0.
	 */
	public void setAlignTicks(boolean alignTicks) {
		setAttr(Attrs.alignTicks, alignTicks);
	}

	/**
	 * Returns the overall animation for all chart updating.
	 * <p>
	 * Default: an instance of {@link Animation}
	 */
	public Animation getAnimation() {
		if (containsKey(Attrs.animation)) {
			return (Animation) getAttr(Attrs.animation);
		} else {
			return (Animation) getAttr(Attrs.animation,
					org.zkoss.chart.Animation.class).asValue();
		}
	}

	/**
	 * Returns whether enable the animation
	 * <p>
	 * Default: true
	 */
	public boolean isAnimation() {
		return getAnimation() != Animation.NONE;
	}

	/**
	 * Animation can be disabled throughout the chart by setting it to false
	 * here. It can be overridden for each individual API method as a function
	 * parameter. The only animation not affected by this option is the initial
	 * series animation, see {@link SeriesPlotOptions#setAnimation(boolean)}.
	 * <p>
	 * The animation can either be set as a boolean or a configuration object.
	 * See {@link #setAnimation(Animation)}. If <code>true</code>, it will use
	 * the 'swing' jQuery easing and a duration of 500 ms. If used as an
	 * Animation object, the following properties are supported:
	 * </p>
	 * <dl>
	 * <dt>duration</dt>
	 * <dd>The duration of the animation in milliseconds.</dd>
	 * <dt>easing</dt>
	 * <dd>When using jQuery as the general framework, the easing can be set to
	 * <code>linear</code> or <code>swing</code>. More easing functions are
	 * available with the use of jQuery plug-ins, most notably the jQuery UI
	 * suite. See <a href="http://api.jquery.com/animate/">the jQuery docs</a>.
	 * When using MooTools as the general framework, use the property name
	 * <code>transition</code> instead of <code>easing</code>.</dd>
	 * </dl>
	 * <p>
	 * 
	 * @see #setAnimation(Animation)
	 * @see Animation
	 */
	public void setAnimation(boolean animation) {
		setAnimation(animation ? new Animation() : Animation.NONE);
	}

	/**
	 * Animation can be disabled throughout the chart by setting it to false by
	 * the method of {@link #setAnimation(boolean)}. It can be overridden for
	 * each individual API method as a function parameter. The only animation
	 * not affected by this option is the initial series animation, see
	 * {@link SeriesPlotOptions#setAnimation(boolean)}.
	 * <p>
	 * Properties are supported:
	 * <dl>
	 * <dt>duration</dt>
	 * <dd>The duration of the animation in milliseconds.</dd>
	 * <dt>easing</dt>
	 * <dd>When using jQuery as the general framework, the easing can be set to
	 * <code>linear</code> or <code>swing</code>. More easing functions are
	 * available with the use of jQuery plug-ins, most notably the jQuery UI
	 * suite. See <a href="http://api.jquery.com/animate/">the jQuery docs</a>.
	 * When using MooTools as the general framework, use the property name
	 * <code>transition</code> instead of <code>easing</code>.</dd>
	 * </dl>
	 * <p>
	 * 
	 * @see #setAnimation(boolean)
	 * @see Animation
	 */
	public void setAnimation(Animation animation) {
		setAttr(Attrs.animation, animation == null ? Animation.NONE
				: animation);
	}

	/**
	 * Returns the background color or gradient for the outer chart area.
	 * <p>
	 * Default: "#FFFFFF".
	 */
	public Color getBackgroundColor() {
		if (!containsKey(Attrs.backgroundColor)) {
			setBackgroundColor("#A0A0A0");
		}
		return (Color) getAttr(Attrs.backgroundColor);
	}

	/**
	 * Sets the background color or gradient for the outer chart area.
	 */
	public void setBackgroundColor(Color color) {
		setAttr(Attrs.backgroundColor, color, NOT_NULL_VALUE);
	}

	/**
	 * Sets the background color or gradient for the outer chart area.
	 */
	public void setBackgroundColor(String color) {
		setBackgroundColor(new Color(color));
	}

	/**
	 * Sets the background color or gradient for the outer chart area.
	 */
	public void setBackgroundColor(LinearGradient color) {
		setBackgroundColor(new Color(color));
	}

	/**
	 * Sets the background color or gradient for the outer chart area.
	 */
	public void setBackgroundColor(RadialGradient color) {
		setBackgroundColor(new Color(color));
	}

	/**
	 * Returns the color of the outer chart border. The border is painted using
	 * vector graphic techniques to allow rounded corners.
	 * <p>
	 * Default: "#4572A7".
	 */
	public Color getBorderColor() {
		if (!containsKey(Attrs.borderColor)) {
			setBorderColor("#4572A7");
		}
		return (Color) getAttr(Attrs.borderColor);
	}

	/**
	 * Sets the color of the outer chart border. The border is painted using
	 * vector graphic techniques to allow rounded corners.
	 */
	public void setBorderColor(Color color) {
		setAttr(Attrs.borderColor, color, NOT_NULL_VALUE);
	}

	/**
	 * Sets the color of the outer chart border. The border is painted using
	 * vector graphic techniques to allow rounded corners.
	 */
	public void setBorderColor(String color) {
		setBorderColor(new Color(color));
	}

	/**
	 * Sets the color of the outer chart border. The border is painted using
	 * vector graphic techniques to allow rounded corners.
	 */
	public void setBorderColor(LinearGradient color) {
		setBorderColor(new Color(color));
	}

	/**
	 * Sets the color of the outer chart border. The border is painted using
	 * vector graphic techniques to allow rounded corners.
	 */
	public void setBorderColor(RadialGradient color) {
		setBorderColor(new Color(color));
	}

	/**
	 * Returns the corner radius of the outer chart border. In export, the
	 * radius defaults to 0.
	 * <p>
	 * Default: 5.
	 */
	public Number getBorderRadius() {
		return getAttr(Attrs.borderRadius, 5).asNumber();
	}

	/**
	 * Sets the corner radius of the outer chart border. In export, the radius
	 * defaults to 0.
	 * <p>
	 * Default: 5.
	 * 
	 * @param borderRadius
	 *            the corner radius of the outer chart border
	 */
	public void setBorderRadius(Number borderRadius) {
		setAttr(Attrs.borderRadius, borderRadius, 5);
	}

	/**
	 * Returns the pixel width of the outer chart border. The border is painted
	 * using vector graphic techniques to allow rounded corners.
	 * <p>
	 * Default: 0.
	 */
	public Number getBorderWidth() {
		return getAttr(Attrs.borderWidth, 0).asNumber();
	}

	/**
	 * Sets the pixel width of the outer chart border. The border is painted
	 * using vector graphic techniques to allow rounded corners.
	 * 
	 * @param borderWidth
	 *            the pixel width of the outer chart border
	 */
	public void setBorderWidth(Number borderWidth) {
		setAttr(Attrs.borderWidth, borderWidth, 0);
	}

	/**
	 * Returns a CSS class name to apply to the charts container div,
	 * allowing unique CSS styling for each chart.
	 * @since 7.2.1.0
	 */
	public String getClassName() {
		return getAttr(Attrs.className, null).asString();
	}

	/**
	 * Sets a CSS class name to apply to the charts container div,
	 * allowing unique CSS styling for each chart.
	 * @since 7.2.1.0
	 */
	public void setClassName(String className) {
		setAttr(Attrs.className, className);
	}

	/**
	 * Returns a map of client side event listeners for the chart.
	 * @since 7.2.2.0
	 */
	public Map<String, JavaScriptValue> getEventsMap() {
		Map<String, JavaScriptValue> map = Generics.cast(getAttr(Attrs.events, null).asValue());
		if (map == null) {
			map = new LinkedHashMap<String, JavaScriptValue>();
			setEvents(map);
		}
		return Generics.cast(getAttr(Attrs.events, null).asValue());
	}

	/**
	 * Sets a map of client side event listeners for the chart.
	 * @since 7.2.2.0
	 */
	public void setEvents(Map<String, JavaScriptValue> events) {
		setAttr(Attrs.events, events, NOT_NULL_VALUE);
	}

	/**
	 * Sets a client side event for the chart.
	 * @since 7.2.2.0
	 */
	@SuppressWarnings("nls")
	public void addEvent(String name, JavaScriptValue event) {
		if ("drilldown".equals(name) || "drillup".equals(name) || 
				"click".equals(name) || "selection".equals(name))
			throw new IllegalArgumentException("Implemented as server side event, can't change: [" + name + "]");

		if (!("addSeries".equals(name) || "afterPrint".equals(name) || 
				"beforePrint".equals(name) || "drillupall".equals(name) || 
				"exportData".equals(name) || "load".equals(name) || 
				"redraw".equals(name) || "render".equals(name)))
			throw new IllegalArgumentException("Unsupported event type: [" + name + "]");

		getEventsMap().put(name, event);
	}

	/**
	 * Returns an explicit height for the chart. By default the height is
	 * calculated from the offset height of the containing element, or 400
	 * pixels if the containing element's height is 0.
	 * <p>
	 * Default: null.
	 */
	public Number getHeight() {
		return getAttr(Attrs.height, null).asNumber();
	}

	/**
	 * Sets an explicit height for the chart. By default the height is
	 * calculated from the offset height of the containing element, or 400
	 * pixels if the containing element's height is 0.
	 */
	public void setHeight(Number height) {
		setAttr(Attrs.height, height);
	}

	/**
	 * Sets an explicit height for the chart.
	 * If given a percentage string (for example '56%'), the height is given as
	 * the percentage of the actual chart width.
	 * This allows for preserving the aspect ratio across responsive sizes.
	 * @since 7.2.1.0
	 */
	public void setHeight(String height) {
		setAttr(Attrs.height, height);
	}

	/**
	 * Returns if true, the axes will scale to the remaining visible series once
	 * one series is hidden. If false, hiding and showing a series will not
	 * affect the axes or the other series. For stacks, once one series within
	 * the stack is hidden, the rest of the stack will close in around it even
	 * if the axis is not affected.
	 * <p>
	 * Default: true.
	 */
	public boolean isIgnoreHiddenSeries() {
		return getAttr(Attrs.ignoreHiddenSeries, true).asBoolean();
	}

	/**
	 * Sets to ignore the hidden series, if true, the axes will scale to the
	 * remaining visible series once one series is hidden. If false, hiding and
	 * showing a series will not affect the axes or the other series. For
	 * stacks, once one series within the stack is hidden, the rest of the stack
	 * will close in around it even if the axis is not affected.
	 * 
	 * @param ignoreHiddenSeries
	 *            if true, the axes will scale to the remaining visible series
	 *            once one series is hidden
	 */
	public void setIgnoreHiddenSeries(boolean ignoreHiddenSeries) {
		setAttr(Attrs.ignoreHiddenSeries, ignoreHiddenSeries);
	}

	/**
	 * Returns whether to invert the axes so that the x axis is vertical and y
	 * axis is horizontal. When true, the x axis is reversed by default. If a
	 * bar series is present in the chart, it will be inverted automatically.
	 * <p>
	 * Default: false.
	 */
	public boolean isInverted() {
		return getAttr(Attrs.inverted, false).asBoolean();
	}

	/**
	 * Sets whether to invert the axes so that the x axis is vertical and y axis
	 * is horizontal. When true, the x axis is reversed by default. If a bar
	 * series is present in the chart, it will be inverted automatically.
	 * 
	 * @param inverted
	 *            whether to invert the axes so that the x axis is vertical and
	 *            y axis is horizontal
	 */
	public void setInverted(boolean inverted) {
		setAttr(Attrs.inverted, inverted);
	}

	/**
	 * Returns the margin between the outer edge of the chart and the plot area.
	 * The numbers in the array designate top, right, bottom and left
	 * respectively. Use the methods of {@link #setMarginTop(Number)},
	 * {@link #setMarginRight(Number)}, {@link #setMarginBottom(Number)} and
	 * {@link #setMarginLeft(Number)} for shorthand setting of one option.
	 * <p>
	 * Default: null.
	 */
	public Number[] getMargin() {
		return getAttr(Attrs.margin, null).asNumberArray();
	}

	/**
	 * Sets the margin between the outer edge of the chart and the plot area.
	 * The numbers in the array designate top, right, bottom and left
	 * respectively. Use the methods of {@link #setMarginTop(Number)},
	 * {@link #setMarginRight(Number)}, {@link #setMarginBottom(Number)} and
	 * {@link #setMarginLeft(Number)} for shorthand setting of one option.
	 * <p>
	 * Default: null.
	 */
	public void setMargin(Number[] margin) {		
		setAttr(Attrs.margin, margin);
	}
	
	/**
	 * Sets the margin between the outer edge of the chart and the plot area.
	 * 
	 */
	public void setMargin(Number margin) {
		setAttr(Attrs.margin, margin);
	}

	/**
	 * Returns the margin between the bottom outer edge of the chart and the
	 * plot area.
	 * <p>
	 * Default: null.
	 */
	public Number getMarginBottom() {
		return getAttr(Attrs.marginBottom, null).asNumber();
	}

	/**
	 * Sets the margin between the bottom outer edge of the chart and the plot
	 * area. Use this to set a fixed pixel value for the margin as opposed to
	 * the default dynamic margin. See also {@link #setSpacingBottom(Number)}.
	 * 
	 * @param marginBottom
	 *            the margin between the bottom outer edge of the chart and the
	 *            plot area
	 */
	public void setMarginBottom(Number marginBottom) {
		setAttr(Attrs.marginBottom, marginBottom);
	}

	/**
	 * Returns the margin between the left outer edge of the chart and the plot
	 * area.
	 * <p>
	 * Default: null.
	 */
	public Number getMarginLeft() {
		return getAttr(Attrs.marginLeft, null).asNumber();
	}

	/**
	 * Sets the margin between the left outer edge of the chart and the plot
	 * area. Use this to set a fixed pixel value for the margin as opposed to
	 * the default dynamic margin. See also {@link #setSpacingLeft(Number)}
	 * 
	 * @param marginLeft
	 *            the margin between the left outer edge of the chart and the
	 *            plot area
	 */
	public void setMarginLeft(Number marginLeft) {
		setAttr(Attrs.marginLeft, marginLeft);
	}

	/**
	 * Returns the margin between the right outer edge of the chart and the plot
	 * area.
	 * <p>
	 * Default: null
	 */
	public Number getMarginRight() {
		return getAttr(Attrs.marginRight, null).asNumber();
	}

	/**
	 * Sets the margin between the right outer edge of the chart and the plot
	 * area. Use this to set a fixed pixel value for the margin as opposed to
	 * the default dynamic margin. See also {@link #setSpacingRight(Number)}.
	 * 
	 * @param marginRight
	 *            the margin between the right outer edge of the chart and the
	 *            plot area
	 */
	public void setMarginRight(Number marginRight) {
		setAttr(Attrs.marginRight, marginRight);
	}

	/**
	 * Returns the margin between the top outer edge of the chart and the plot
	 * area.
	 * <p>
	 * Default: null.
	 */
	public Number getMarginTop() {
		return getAttr(Attrs.marginTop, null).asNumber();
	}

	/**
	 * Sets the margin between the top outer edge of the chart and the plot
	 * area. Use this to set a fixed pixel value for the margin as opposed to
	 * the default dynamic margin. See also {@link #setSpacingTop(Number)}.
	 * 
	 * @param marginTop
	 *            the margin between the top outer edge of the chart and the
	 *            plot area
	 */
	public void setMarginTop(Number marginTop) {
		setAttr(Attrs.marginTop, marginTop);
	}
	
	/**
	 * Returns the options to render charts in 3 dimensions.
	 * <p>
	 * Default: null.
	 */
	public Options3D getOptions3D() {
		Options3D options3d = (Options3D) getAttr(Attrs.options3d);
		if (options3d == null) {
			options3d = new Options3D();
			setOptions3D(options3d);
		}
		return options3d;
	}

	/**
	 * Sets the options to render charts in 3 dimensions.
	 * 
	 * @param options3D
	 *            the options to render charts in 3 dimensions.
	 */
	public void setOptions3D(Options3D options3D) {
		setAttr(Attrs.options3d, options3D);
	}

	/**
	 * Returns setting a key to switch between zooming and panning.
	 * Can be one of alt, ctrl, meta (the command key on Mac and Windows key on Windows) or shift.
	 * The keys are mapped directly to the key properties of the click event argument (event.altKey, event.ctrlKey, event.metaKey and event.shiftKey).
	 * <p>
	 * Default: null
	 * @since 7.2.1.0
	 */
	public String getPanKey() {
		return getAttr(Attrs.panKey, null).asString();
	}

	/**
	 * Sets setting a key to switch between zooming and panning.
	 * Can be one of alt, ctrl, meta (the command key on Mac and Windows key on Windows) or shift.
	 * The keys are mapped directly to the key properties of the click event argument (event.altKey, event.ctrlKey, event.metaKey and event.shiftKey).
	 * @since 7.2.1.0
	 */
	public void setPanKey(String panKey) {
		if (!("alt".equals(panKey) || "ctrl".equals(panKey) || "meta".equals(panKey) || "shift".equals(panKey)))
			throw new IllegalArgumentException("Unsupported panKey: [" + panKey + "]");
		setAttr(Attrs.panKey, panKey);
	}

	/**
	 * Returns if allow panning in a chart.
	 * Best used with panKey to combine zooming and panning.
	 *
	 * On touch devices, when the tooltip.followTouchMove option is true (default),
	 * panning requires two fingers. To allow panning with one finger, set followTouchMove to false.
	 * <p>
	 * Default: false
	 * @since 7.2.1.0
	 */
	public boolean isPanning() {
		return getAttr(Attrs.panning, false).asBoolean();
	}

	/**
	 * Sets if allow panning in a chart.
	 * Best used with panKey to combine zooming and panning.
	 *
	 * On touch devices, when the tooltip.followTouchMove option is true (default),
	 * panning requires two fingers. To allow panning with one finger, set followTouchMove to false.
	 * @since 7.2.1.0
	 */
	public void setPanning(boolean panning) {
		setAttr(Attrs.panning, panning);
	}

	/**
	 * Returns common options for all yAxes rendered in a parallel coordinates plot, if any.
	 * Otherwise, create a new one.
	 * @since 7.2.1.0
	 */
	public ParallelAxes getParallelAxes() {
		ParallelAxes parallelAxes = (ParallelAxes) getAttr(Attrs.parallelAxes);
		if (parallelAxes == null) {
			parallelAxes = new ParallelAxes();
			setParallelAxes(parallelAxes);
		}
		return parallelAxes;
	}

	/**
	 * Sets common options for all yAxes rendered in a parallel coordinates plot.
	 * @since 7.2.1.0
	 */
	public void setParallelAxes(ParallelAxes parallelAxes) {
		setAttr(Attrs.parallelAxes, parallelAxes);
	}

	/**
	 * Returns a flag to render charts as a parallel coordinates plot.
	 * In a parallel coordinates plot (||-coords) by default all required yAxes
	 * are generated and the legend is disabled.
	 * <p>
	 * Default: false
	 * @since 7.2.1.0
	 */
	public boolean isParallelCoordinates() {
		return getAttr(Attrs.parallelCoordinates, false).asBoolean();
	}

	/**
	 * Sets a flag to render charts as a parallel coordinates plot.
	 * In a parallel coordinates plot (||-coords) by default all required yAxes
	 * are generated and the legend is disabled.
	 * @since 7.2.1.0
	 */
	public void setParallelCoordinates(boolean parallelCoordinates) {
		setAttr(Attrs.parallelCoordinates, parallelCoordinates);
	}

	/**
	 * Returns equivalent to {@link #getZoomType()}, but for multitouch
	 * gestures only. By default, the <code>pinchType</code> is the same as the
	 * <code>zoomType</code> setting. However, pinching can be enabled
	 * separately in some cases, for example in stock charts where a mouse drag
	 * pans the chart, while pinching is enabled.
	 * <p>
	 * Default: null.
	 */
	public String getPinchType() {
		return getAttr(Attrs.pinchType, null).asString();
	}

	/**
	 * Sets equivalent to {@link #setZoomType(String)}, but for multitouch
	 * gestures only. By default, the <code>pinchType</code> is the same as the
	 * <code>zoomType</code> setting. However, pinching can be enabled
	 * separately in some cases, for example in stock charts where a mouse drag
	 * pans the chart, while pinching is enabled.
	 * <p>
	 * Default: null.
	 * 
	 * @see #setZoomType(String)
	 */
	public void setPinchType(String pinchType) {
		setAttr(Attrs.pinchType, pinchType);
	}

	/**
	 * Returns the background color for the plot area.
	 * <p>
	 * Default: null.
	 */
	public Color getPlotBackgroundColor() {
		return (Color) getAttr(Attrs.plotBackgroundColor, null).asValue();
	}

	/**
	 * Sets the background color for the plot area.
	 */
	public void setPlotBackgroundColor(Color color) {
		setAttr(Attrs.plotBackgroundColor, color);
	}

	/**
	 * Sets the background color for the plot area.
	 */
	public void setPlotBackgroundColor(String color) {
		setPlotBackgroundColor(new Color(color));
	}

	/**
	 * Sets the background color for the plot area.
	 */
	public void setPlotBackgroundColor(LinearGradient color) {
		setPlotBackgroundColor(new Color(color));
	}

	/**
	 * Sets the background color for the plot area.
	 */
	public void setPlotBackgroundColor(RadialGradient color) {
		setPlotBackgroundColor(new Color(color));
	}
	
	/**
	 * Returns the URL for an image to use as the plot background.
	 * <p>
	 * Default: null.
	 */
	public String getPlotBackgroundImage() {
		return getAttr(Attrs.plotBackgroundImage, null).asString();
	}

	/**
	 * Sets the URL for an image to use as the plot background. To set an image
	 * as the background for the entire chart, set a CSS background image to the
	 * container element. Note that for the image to be applied to exported
	 * charts, its URL needs to be accessible by the export server.
	 * 
	 * @param plotBackgroundImage
	 *            the URL for an image to use as the plot background
	 */
	public void setPlotBackgroundImage(String plotBackgroundImage) {
		setAttr(Attrs.plotBackgroundImage, plotBackgroundImage);
	}

	/**
	 * Returns the color of the inner chart or plot area border.
	 * <p>
	 * Default: "#C0C0C0".
	 */
	public Color getPlotBorderColor() {
		if (!containsKey(Attrs.plotBorderColor)) {
			setPlotBorderColor("#C0C0C0");
		}
		return (Color) getAttr(Attrs.plotBorderColor);
	}

	/**
	 * Sets the color of the inner chart or plot area border.
	 */
	public void setPlotBorderColor(Color color) {
		setAttr(Attrs.plotBorderColor, color, NOT_NULL_VALUE);
	}

	/**
	 * Sets the color of the inner chart or plot area border.
	 */
	public void setPlotBorderColor(String color) {
		setPlotBorderColor(new Color(color));
	}

	/**
	 * Sets the color of the inner chart or plot area border.
	 */
	public void setPlotBorderColor(LinearGradient color) {
		setPlotBorderColor(new Color(color));
	}

	/**
	 * Sets the color of the inner chart or plot area border.
	 */
	public void setPlotBorderColor(RadialGradient color) {
		setPlotBorderColor(new Color(color));
	}

	/**
	 * Returns the pixel width of the plot area border.
	 * <p>
	 * Default: 0.
	 */
	public Number getPlotBorderWidth() {
		return getAttr(Attrs.plotBorderWidth, 0).asNumber();
	}

	/**
	 * Sets the pixel width of the plot area border.
	 * 
	 * @param plotBorderWidth
	 *            the pixel width of the plot area border
	 */
	public void setPlotBorderWidth(Number plotBorderWidth) {
		setAttr(Attrs.plotBorderWidth, plotBorderWidth, 0);
	}

	/**
	 * Returns whether to apply a drop shadow to the plot area. Requires that
	 * {@link #setPlotBackgroundColor(String)} be set.
	 * <p>
	 * Default: false.
	 */
	public boolean isPlotShadow() {
		return getAttr(Attrs.plotShadow, false).asBoolean();
	}

	/**
	 * Sets whether to apply a drop shadow to the plot area. Requires that
	 * {@link #setPlotBackgroundColor(String)} be set.
	 * 
	 * @param plotShadow
	 *            whether to apply a drop shadow to the plot area
	 */
	public void setPlotShadow(boolean plotShadow) {
		setAttr(Attrs.plotShadow, plotShadow);
	}

	/**
	 * Returns whether transforms into the polar coordinate system
	 * <p>
	 * Default: false.
	 */
	public boolean isPolar() {
		return getAttr(Attrs.polar, false).asBoolean();
	}

	/**
	 * Sets to true to enable the cartesian charts like line, spline, area and
	 * column are transformed into the polar coordinate system.
	 * 
	 * @param polar
	 *            when true, cartesian charts like line, spline, area and column
	 *            are transformed into the polar coordinate system
	 */
	public void setPolar(boolean polar) {
		setAttr(Attrs.polar, polar);
	}

	/**
	 * Returns whether to reflow the chart to fit the width of the container div
	 * on resizing the window.
	 * <p>
	 * Default: true.
	 */
	public boolean isReflow() {
		return getAttr(Attrs.reflow, true).asBoolean();
	}

	/**
	 * Sets whether to reflow the chart to fit the width of the container div on
	 * resizing the window.
	 * 
	 * @param reflow
	 *            whether to reflow the chart to fit the width of the container
	 *            div on resizing the window
	 */
	public void setReflow(boolean reflow) {
		setAttr(Attrs.reflow, reflow);
	}

	/**
	 * Returns the button that appears after a selection zoom, allowing the user
	 * to reset zoom.
	 * <p>
	 * Default: null.
	 */
	public ResetZoomButton getResetZoomButton() {
		return (ResetZoomButton) getAttr(Attrs.resetZoomButton, null).asValue();
	}

	/**
	 * Sets the button that appears after a selection zoom, allowing the user to
	 * reset zoom.
	 * 
	 * @param resetZoomButton
	 *            the button that appears after a selection zoom, allowing the
	 *            user to reset zoom
	 */
	public void setResetZoomButton(ResetZoomButton resetZoomButton) {
		setAttr(Attrs.resetZoomButton, resetZoomButton);
	}

	/**
	 * Returns the options for a scrollable plot area.
	 * <p>
	 * @since 7.2.2.0
	 */
	public ScrollablePlotArea getScrollablePlotArea() {
		ScrollablePlotArea scrollPlotArea = (ScrollablePlotArea) getAttr(Attrs.scrollablePlotArea);
		if (scrollPlotArea == null) {
			scrollPlotArea = new ScrollablePlotArea();
			setScrollablePlotArea(scrollPlotArea);
		}
		return scrollPlotArea;
	}

	/**
	 * Sets the options for a scrollable plot area.
	 * @since 7.2.2.0
	 */
	public void setScrollablePlotArea(ScrollablePlotArea scrollablePlotArea) {
		setAttr(Attrs.scrollablePlotArea, scrollablePlotArea);
	}

	/**
	 * Returns the background color of the marker square when selecting (zooming
	 * in on) an area of the chart.
	 * <p>
	 * Default: "rgba(69,114,167,0.25)".
	 */
	public Color getSelectionMarkerFill() {
		if (!containsKey(Attrs.selectionMarkerFill)) {
			setSelectionMarkerFill("rgba(69,114,167,0.25)");
		}
		return (Color) getAttr(Attrs.selectionMarkerFill);
	}

	/**
	 * Sets the background color of the marker square when selecting (zooming in
	 * on) an area of the chart.
	 */
	public void setSelectionMarkerFill(Color color) {
		setAttr(Attrs.selectionMarkerFill, color, NOT_NULL_VALUE);
	}

	/**
	 * Sets the background color of the marker square when selecting (zooming in
	 * on) an area of the chart.
	 */
	public void setSelectionMarkerFill(String color) {
		setSelectionMarkerFill(new Color(color));
	}

	/**
	 * Sets the background color of the marker square when selecting (zooming in
	 * on) an area of the chart.
	 */
	public void setSelectionMarkerFill(LinearGradient color) {
		setSelectionMarkerFill(new Color(color));
	}

	/**
	 * Sets the background color of the marker square when selecting (zooming in
	 * on) an area of the chart.
	 */
	public void setSelectionMarkerFill(RadialGradient color) {
		setSelectionMarkerFill(new Color(color));
	}

	/**
	 * Returns whether to apply a drop shadow to the outer chart area. Requires
	 * that backgroundColor be set.
	 * <p>
	 * Default: false.
	 * 
	 * @see Shadow
	 */
	public boolean isShadow() {
		return getShadow() != Shadow.NONE;
	}

	/**
	 * Returns whether to apply a drop shadow to the outer chart area. Requires
	 * that backgroundColor be set.
	 * <p>
	 * Default: {@link Shadow#NONE}.
	 * 
	 * @see Shadow
	 */
	public Shadow getShadow() {
		return (Shadow) getAttr(Attrs.shadow, Shadow.NONE).asValue();
	}

	/**
	 * Sets whether to apply a drop shadow to the outer chart area. Requires
	 * that backgroundColor be set.
	 */
	public void setShadow(boolean shadow) {
		setShadow(shadow ? new Shadow() : Shadow.NONE);
	}

	/**
	 * Sets whether to apply a drop shadow to the outer chart area. Requires
	 * that backgroundColor be set.
	 * 
	 * @see Shadow
	 */
	public void setShadow(Shadow shadow) {
		setAttr(Attrs.shadow, shadow);
	}

	/**
	 * Returns whether to show the axes initially. This only applies to empty
	 * charts where series are added dynamically, as axes are automatically
	 * added to cartesian series.
	 * <p>
	 * Default: false.
	 */
	public boolean isShowAxes() {
		return getAttr(Attrs.showAxes, false).asBoolean();
	}

	/**
	 * Sets whether to show the axes initially. This only applies to empty
	 * charts where series are added dynamically, as axes are automatically
	 * added to cartesian series.
	 * 
	 * @param showAxes
	 *            whether to show the axes initially
	 */
	public void setShowAxes(boolean showAxes) {
		setAttr(Attrs.showAxes, showAxes);
	}

	/**
	 * Returns the distance between the outer edge of the chart and the content,
	 * like title, legend, axis title or labels. The numbers in the array
	 * designate top, right, bottom and left respectively.
	 * <p>
	 * Default: [10, 10, 15, 10].
	 */
	public Number[] getSpacing() {
		return getAttr(Attrs.spacing, new Number[] { 10, 10, 15, 10 })
				.asNumberArray();
	}

	/**
	 * Sets the distance between the outer edge of the chart and the content,
	 * like title, legend, axis title or labels. The numbers in the array
	 * designate top, right, bottom and left respectively. Use the methods of
	 * {@link #setSpacingTop(Number)}, {@link #setSpacingRight(Number)},
	 * {@link #setSpacingBottom(Number)} and {@link #setSpacingLeft(Number)} for
	 * shorthand setting of one option.
	 * 
	 * @param spacing
	 *            the distance between the outer edge of the chart and the
	 *            content, like title, legend, axis title or labels
	 */
	public void setSpacing(Number[] spacing) {
		setAttr(Attrs.spacing, spacing, NOT_NULL_VALUE);
	}

	/**
	 * Returns the space between the bottom edge of the chart and the content
	 * (plot area, axis title and labels, title, subtitle or legend in top
	 * position).
	 * <p>
	 * Default: 15.
	 */
	public Number getSpacingBottom() {
		return getAttr(Attrs.spacingBottom, 15).asNumber();
	}

	/**
	 * Sets the space between the bottom edge of the chart and the content (plot
	 * area, axis title and labels, title, subtitle or legend in top position).
	 * </p>
	 * 
	 * @param spacingBottom
	 *            The space between the bottom edge of the chart and the content
	 *            (plot area, axis title and labels, title, subtitle or legend
	 *            in top position)
	 */
	public void setSpacingBottom(Number spacingBottom) {
		setAttr(Attrs.spacingBottom, spacingBottom, 15);
	}

	/**
	 * Returns the space between the left edge of the chart and the content
	 * (plot area, axis title and labels, title, subtitle or legend in top
	 * position).
	 * <p>
	 * Default: 10.
	 */
	public Number getSpacingLeft() {
		return getAttr(Attrs.spacingLeft, 10).asNumber();
	}

	/**
	 * Sets the space between the left edge of the chart and the content (plot
	 * area, axis title and labels, title, subtitle or legend in top position).
	 * </p>
	 * 
	 * @param spacingLeft
	 *            The space between the left edge of the chart and the content
	 *            (plot area, axis title and labels, title, subtitle or legend
	 *            in top position)
	 */
	public void setSpacingLeft(Number spacingLeft) {
		setAttr(Attrs.spacingLeft, spacingLeft, 10);
	}

	/**
	 * Returns the space between the right edge of the chart and the content
	 * (plot area, axis title and labels, title, subtitle or legend in top
	 * position).
	 * <p>
	 * Default: 10.
	 */
	public Number getSpacingRight() {
		return getAttr(Attrs.spacingRight, 10).asNumber();
	}

	/**
	 * Sets the space between the right edge of the chart and the content (plot
	 * area, axis title and labels, title, subtitle or legend in top position).
	 * 
	 * @param spacingRight
	 *            The space between the right edge of the chart and the content
	 *            (plot area, axis title and labels, title, subtitle or legend
	 *            in top position)
	 */
	public void setSpacingRight(Number spacingRight) {
		setAttr(Attrs.spacingRight, spacingRight, 10);
	}

	/**
	 * Returns the space between the top edge of the chart and the content (plot
	 * area, axis title and labels, title, subtitle or legend in top position).
	 * <p>
	 * Default: 10.
	 */
	public Number getSpacingTop() {
		return getAttr(Attrs.spacingTop, 10).asNumber();
	}

	/**
	 * Sets the space between the top edge of the chart and the content (plot
	 * area, axis title and labels, title, subtitle or legend in top position).
	 * 
	 * @param spacingTop
	 *            The space between the top edge of the chart and the content
	 *            (plot area, axis title and labels, title, subtitle or legend
	 *            in top position)
	 */
	public void setSpacingTop(Number spacingTop) {
		setAttr(Attrs.spacingTop, spacingTop, 10);
	}

	/**
	 * Returns additional CSS styles to apply inline to the container
	 * <code>div</code>. Note that since the default font styles are applied in
	 * the renderer, it is ignorant of the individual chart options and must be
	 * set globally. Defaults to:
	 * <pre>
	 * fontFamily: "Lucida Grande", "Lucida Sans Unicode", Verdana, Arial, Helvetica, sans-serif; fontSize: 12px
	 * </pre>
	 * @param <K> type for the key
	 * <p>
	 * if style was set using {@link #setStyle(String)}, type will be String
	 * <p>
	 * if style was set using {@link #setStyle(Map)}, type will be the same as the type from the key of the map
	 * <p>
	 * if style was not set, default style will be applied, and type will be String
	 * @param <V> type for the value
	 * <p>
	 * if style was set using {@link #setStyle(String)}, type will be String
	 * <p>
	 * if style was set using {@link #setStyle(Map)}, type will be the same as the type from the key of the map
	 * <p>
	 * if style was not set, default style will be applied, and type will be String
	 */
	public <K, V> Map<K, V> getStyle() {
		if (!containsKey(Attrs.style)) {
			setStyle("fontFamily: \"Lucida Grande\", \"Lucida Sans Unicode\", Verdana, Arial, Helvetica, sans-serif; fontSize: 12px");
		}
		return Generics.cast(getAttr(Attrs.style));
	}

	/**
	 * Returns whether to apply styled mode. When in styled mode, no presentational
	 * attributes or CSS are applied to the chart SVG. Instead,
	 * CSS rules are required to style the chart. The default style sheet is available from
	 * <a href="https://code.highcharts.com/css/highcharts.css">highcharts.css</a>.
	 * <p>Default: {@code false}
	 * @since 10.2.1.0
	 */
	public boolean isStyledMode() {
		return getAttr(Attrs.styledMode, false).asBoolean();
	}

	/**
	 * Sets whether to apply styled mode. When in styled mode, no presentational
	 * attributes or CSS are applied to the chart SVG. Instead,
	 * CSS rules are required to style the chart. The default style sheet is available from
	 * <a href="https://code.highcharts.com/css/highcharts.css">highcharts.css</a>.
	 * <p>Default: {@code false}
	 * @since 10.2.1.0
	 */
	public void setStyledMode(boolean styledMode) {
		setAttr(Attrs.styledMode, styledMode);
	}

	/**
	 * Sets additional CSS styles to apply inline to the container
	 * <code>div</code>. Note that since the default font styles are applied in
	 * the renderer, it is ignorant of the individual chart options and must be
	 * set globally.
	 * <p>
	 * This method allows <b>simple</b> settings like
	 * <pre>{@code
	 * label.setStyle("fontWeight: 'bold'; color: 'white'; textShadow: '0px 1px 2px black'");
	 * }</pre>
	 * <p>
	 * In the format of CSS key:value, and ";" separating multiple key:value pairs,
	 * if you have more a complex CSS styling, we recommend using {@link #setStyle(Map)}
	 * @param style one or many CSS key:value pairs in a single line string format
	 * @see #setStyle(Map)
	 */
	public void setStyle(String style) {
		setStyle(MapsHelper.parse(new LinkedHashMap<String, String>(), style, ':', ';', '\''));
	}

	/**
	 * Sets additional CSS styles to apply inline to the container
	 * <code>div</code>. Note that since the default font styles are applied in
	 * the renderer, it is ignorant of the individual chart options and must be
	 * set globally.
	 * <p>
	 * This method allows <b>complex</b> styling like:
	 * <pre>
	 * {@code
	 * Map<String, Object> styles = new HashMap<String, Object>();
	 * styles.put("fill", "#E8E8EA");
	 * styles.put("stroke", "#6E6E78");
	 * styles.put("stroke-width", "1");
	 * styles.put("r", "0");
	 * Map<String, Object> states = new HashMap<String, Object>();
	 * Map<String, String> hover = new HashMap<String, String>();
	 * hover.put("fill", "#F4F4F6");
	 * hover.put("stroke", "#6E6E78");
	 * hover.put("stroke-width", "1");
	 * hover.put("r", "0");
	 * states.put("hover", hover);
	 * styles.put("states", states);
	 * Map<String, String> style = new HashMap<String, String>();
	 * style.put("color", "#4D5663");
	 * style.put("fontSize", "12px");
	 * style.put("fontFamily", "'Arial'");
	 * style.put("fontWeight", "normal");
	 * style.put("padding", "10px");
	 * styles.put("style", style);
	 * }</pre>
	 * which will generate the final results like:
	 * <pre>{@code
	 * {
	 *     "fill": "#E8E8EA",
	 *     "stroke": "#6E6E78",
	 *     "stroke-width": "1",
	 *     "r": "0",
	 *     "states":
	 *     {
	 *         "hover":
	 *         {
	 *             "fill": "#F4F4F6",
	 *             "stroke": "#6E6E78",
	 *             "stroke-width": "1",
	 *             "r": "0"
	 *         }
	 *     },
	 *     "style":
	 *     {
	 *         "color": "#4D5663",
	 *         "fontSize": "12px",
	 *         "fontFamily": "'Arial'",
	 *         "fontWeight": "normal",
	 *         "padding": "10px"
	 *     }
	 * }
	 * }</pre>
	 * @param <K> Type for the key, usually String
	 * @param <V> Type for the value, usually String. For nested maps, use Object or any other type.
	 * @param style the style object
	 */
	public <K, V> void setStyle(Map<K, V> style) {
		setAttr(Attrs.style, style, NOT_NULL_VALUE);
	}

	/**
	 * Returns the default series type for the chart. Can be any of the chart
	 * types listed under {@link Charts}.
	 * <p>
	 * Default: "line".
	 */
	public String getType() {
		return getAttr(Attrs.type, Charts.LINE).asString();
	}

	/**
	 * Sets the default series type for the chart. Can be any of the chart types
	 * listed under {@link Charts}.
	 * 
	 * @param type
	 *            the default series type for the chart
	 */
	public void setType(String type) {
		setAttr(Attrs.type, type, Charts.LINE);
	}

	/**
	 * Returns an explicit width for the chart. By default the width is
	 * calculated from the offset width of the containing element.
	 * <p>
	 * Default: null.
	 */
	public Number getWidth() {
		return getAttr(Attrs.width, null).asNumber();
	}

	/**
	 * Sets an explicit width for the chart. By default the width is calculated
	 * from the offset width of the containing element.
	 * 
	 * @param width
	 *            an explicit width for the chart
	 */
	public void setWidth(Number width) {
		setAttr(Attrs.width, width);
	}

	/**
	 * Returns decides in what dimensions the user can zoom by dragging the
	 * mouse. Can be one of <code>x</code>, <code>y</code> or <code>xy</code>.
	 * <p>
	 * Default: null.
	 */
	public String getZoomType() {
		return getAttr(Attrs.zoomType, null).asString();
	}

	/**
	 * Sets decides in what dimensions the user can zoom by dragging the mouse.
	 * Can be one of <code>x</code>, <code>y</code> or <code>xy</code>.
	 * 
	 * @param zoomType
	 *            decides in what dimensions the user can zoom by dragging the
	 *            mouse
	 */
	public void setZoomType(String zoomType) {
		if (!(zoomType == null || "x".equals(zoomType) || "y".equals(zoomType) || "xy"
				.equals(zoomType)))
			throw new IllegalArgumentException("Unsupported zoomType: ["
					+ zoomType + "]");
		setAttr(Attrs.zoomType, zoomType);
	}
	
	/**
	 * Returns whether selection event will zoom the chart to the selected area.
	 * <p>
	 * Default: true.
	 * 
	 */
	public Boolean isEnableZoomSelection() {
		return getAttr(Attrs.enableZoomSelection, true).asBoolean();
	}
	
	/**
	 * Sets whether selection event will zoom the chart to the selected area.
	 * 
	 */
	public void setEnableZoomSelection(boolean enableZoomSelection) {
		setAttr(Attrs.enableZoomSelection, enableZoomSelection);
	}
}
