/* ClusterLayoutAlgorithm.java

	Purpose:
		
	Description:
		
	History:
		2:27 PM 2022/9/8, Created by jumperchen

Copyright (C) 2022 Potix Corporation. All Rights Reserved.
*/
package org.zkoss.chart;

import org.zkoss.chart.util.DynamicalAttribute;
import org.zkoss.json.JavaScriptValue;

/**
 * Options for layout algorithm. Inside there are options to change the type of
 * the algorithm, gridSize, distance or iterations.
 *
 * <p>
 * All the options in this class support {@link DynamicalAttribute}.
 *
 * @author jumperchen
 * @since 10.2.1.0
 */
public class ClusterLayoutAlgorithm extends Optionable {
	private enum Attrs implements PlotAttribute, DynamicalAttribute {
		distance,
		gridSize,
		iterations,
		kmeansThreshold,
		type
	}

	/**
	 * When {@code type} is set to {@code kmeans}, {@code distance} is a maximum distance between point
	 * and cluster center so that this point will be inside the cluster.
	 * The distance is either a number defining pixels or a percentage defining
	 * a percentage of the plot area width.
	 * <p>Default: {@code 40}
	 */
	public Object getDistance() {
		return getAttr(Attrs.distance, 40).asNumber();
	}

	/**
	 * When {@code type} is set to {@code kmeans}, {@code distance} is a maximum distance between point
	 * and cluster center so that this point will be inside the cluster.
	 * The distance is either a number defining pixels or a percentage defining
	 * a percentage of the plot area width.
	 * <p>Default: {@code 40}
	 * @see #setDistance(Number)
	 */
	public void setDistance(String distance) {
		setAttr(Attrs.distance, distance);
	}

	/**
	 * When {@code type} is set to {@code kmeans}, {@code distance} is a maximum distance between point
	 * and cluster center so that this point will be inside the cluster.
	 * The distance is either a number defining pixels or a percentage defining
	 * a percentage of the plot area width.
	 * <p>Default: {@code 40}
	 * @see #setDistance(String)
	 */
	public void setDistance(Number distance) {
		setAttr(Attrs.distance, distance);
	}


	/**
	 * When {@code type} is set to the {@code grid}, {@code gridSize} is a size of
	 * a grid square element either as a number defining pixels, or a percentage
	 * defining a percentage of the plot area width.
	 * <p>Default: {@code 50}
	 */
	public Object getGridSize() {
		return getAttr(Attrs.gridSize, 50).asNumber();
	}

	/**
	 * When {@code type} is set to the {@code grid}, {@code gridSize} is a size of
	 * a grid square element either as a number defining pixels, or a percentage
	 * defining a percentage of the plot area width.
	 * <p>Default: {@code 50}
	 * @see #setGridSize(Number)
	 */
	public void setGridSize(String gridSize) {
		setAttr(Attrs.gridSize, gridSize);
	}

	/**
	 * When {@code type} is set to the {@code grid}, {@code gridSize} is a size of
	 * a grid square element either as a number defining pixels, or a percentage
	 * defining a percentage of the plot area width.
	 * <p>Default: {@code 50}
	 * @see #setGridSize(String)
	 */
	public void setGridSize(Number gridSize) {
		setAttr(Attrs.gridSize, gridSize);
	}

	/**
	 * When {@code type} is set to {@code kmeans}, {@code iterations} are the number
	 * of iterations that this algorithm will be repeated to find clusters positions.
	 * <p>Default: null
	 */
	public Number getIterations() {
		return getAttr(Attrs.iterations, null).asNumber();
	}

	/**
	 * When {@code type} is set to {@code kmeans}, {@code iterations} are the number
	 * of iterations that this algorithm will be repeated to find clusters positions.
	 * <p>Default: null
	 */
	public void setIterations(Number iterations) {
		setAttr(Attrs.iterations, iterations);
	}

	/**
	 * When {@code type} is set to {@code null} and there are more visible points
	 * than the {@code kmeansThreshold} the {@code grid} algorithm is used to find
	 * clusters, otherwise {@code kmeans}. It ensures good performance on large
	 * datasets and better clusters arrangement after the zoom.
	 * <p>Default: 100
	 */
	public Number getKmeansThreshold() {
		return getAttr(Attrs.kmeansThreshold, 100).asNumber();
	}

	/**
	 * When {@code type} is set to {@code null} and there are more visible points
	 * than the {@code kmeansThreshold} the {@code grid} algorithm is used to find
	 * clusters, otherwise {@code kmeans}. It ensures good performance on large
	 * datasets and better clusters arrangement after the zoom.
	 * <p>Default: 100
	 */
	public void setKmeansThreshold(Number kmeansThreshold) {
		setAttr(Attrs.kmeansThreshold, kmeansThreshold);
	}

	/**
	 * Returns the type of the algorithm used to combine points into a cluster.
	 * <p>Default: null.
	 */
	public Object getType() {
		return getAttr(Attrs.type);
	}

	/**
	 * Sets the type of the algorithm used to combine points into a cluster.
	 * <p>There are three available algorithms:
	 * <ul><li>
	 * {@code grid} - grid-based clustering technique. Points are assigned to squares
	 * of set size depending on their position on the plot area. 
	 * Points inside the grid square are combined into a cluster.
	 * The grid size can be controlled by gridSize property (grid size changes at certain zoom levels).
	 * </li>
	 * <li>
	 * {@code kmeans} - based on K-Means clustering technique.
	 * In the first step, points are divided using the grid method
	 * (distance property is a grid size) to find the initial amount of clusters.
	 * Next, each point is classified by computing the distance between each cluster
	 * center and that point. When the closest cluster distance is lower than distance
	 * property set by a user the point is added to this cluster otherwise is classified
	 * as noise. The algorithm is repeated until each cluster center not change
	 * its previous position more than one pixel. This technique is more accurate
	 * but also more time consuming than the grid algorithm, especially for big datasets.
	 * </li>
	 * <li>
	 * {@code optimizedKmeans} - based on K-Means clustering technique.
	 * This algorithm uses k-means algorithm only on the chart initialization or
	 * when chart extremes have greater range than on initialization.
	 * When a chart is redrawn the algorithm checks only clustered points distance
	 * from the cluster center and rebuild it when the point is spaced enough
	 * to be outside the cluster. It provides performance improvement and more stable
	 * clusters position yet can be used rather on small and sparse datasets.</li>
	 * </ul>
	 *
	 * <p>
	 * By default, the algorithm depends on visible quantity of points and kmeansThreshold.
	 * When there are more visible points than the kmeansThreshold the grid algorithm is used,
	 * otherwise kmeans.
	 * @param type either {@code grid}, {@code kmeans}, and {@code optimizedKmeans}
	 */
	public void setType(String type) {
		setAttr(Attrs.type, type);
	}

	/**
	 * Sets the custom clustering algorithm can be added by assigning a callback
	 * function as the type property. This function takes an array of {@code processedXData},
	 * {@code processedYData}, {@code processedXData} indexes and {@code layoutAlgorithm}
	 * options as arguments and should return an object with grouped data.
	 * <p>
	 * The algorithm should return an object like that:
	 * <pre><code>
	 *{
	 *  clusterId1: [{
	 *      x: 573,
	 *      y: 285,
	 *      index: 1 // point index in the data array
	 *  }, {
	 *      x: 521,
	 *      y: 197,
	 *      index: 2
	 *  }],
	 *  clusterId2: [{
	 *      ...
	 *  }]
	 *  ...
	 * }
	 * }
	 *
	 * </code></pre>
	 * <p> {@code clusterId} (example above - unique id of a cluster or noise) is
	 * an array of points belonging to a cluster. If the array has only one point
	 * or fewer points than set in cluster.minimumClusterSize it won't be combined
	 * into a cluster.
	 * @see #setType(String) 
	 */
	public void setType(JavaScriptValue type) {
		setAttr(Attrs.type, type);
	}
}
