/* DrillUpButton.java

	Purpose:
		
	Description:
		
	History:
		Tue, Jan 28, 2014 12:21:24 PM, Created by RaymondChao

Copyright (C) 2014 Potix Corporation. All Rights Reserved.

 */
package org.zkoss.chart;

import java.util.Map;

import org.zkoss.chart.util.DynamicalAttribute;
import org.zkoss.lang.Generics;

/**
 * Options for the drill up button that appears when drilling down on a series.
 * The text for the button is defined in {@link Lang#getDrillUpText()}.
 * 
 * @author jumperchen
 * @author RaymondChao
 */
public class DrillUpButton extends Optionable {
	private enum Attrs implements PlotAttribute, DynamicalAttribute {
		position, relativeTo, theme
	}

	/**
	 * Returns positioning options for the button within the
	 * <code>relativeTo</code> box. Available properties are <code>x</code>,
	 * <code>y</code>, <code>align</code> and <code>verticalAlign</code>.
	 * <p>
	 * Default: null
	 */
	public Position getPosition() {
		return (Position) getAttr(Attrs.position, null).asValue();
	}

	/**
	 * Sets positioning options for the button within the
	 * <code>relativeTo</code> box. Available properties are <code>x</code>,
	 * <code>y</code>, <code>align</code> and <code>verticalAlign</code>.
	 */
	public void setPosition(Position position) {
		setAttr(Attrs.position, position);
	}

	/**
	 * Returns what box to align the button to. Can be either "plotBox" or
	 * "spacingBox".
	 * <p>
	 * Default: <code>plotBox</code>
	 */
	public String getRelativeTo() {
		return getAttr(Attrs.relativeTo, "plotBox").asString();
	}

	/**
	 * Sets what box to align the button to. Can be either "plotBox" or
	 * "spacingBox".
	 * <p>
	 * Default: <code>plotBox</code>
	 */
	public void setRelativeTo(String relativeTo) {
		if (!("plotBox".equals(relativeTo) || "spacingBox".equals(relativeTo)))
			throw new IllegalArgumentException("Unsupported value: ["
					+ relativeTo + "]");
		setAttr(Attrs.relativeTo, relativeTo, "plotBox");
	}

	/**
	 * Returns a collection of attributes for the button. The object takes SVG
	 * attributes like <code>fill</code>, <code>stroke</code>,
	 * <code>stroke-width</code> or <code>r</code>, the border radius. The theme
	 * also supports <code>style</code>, a collection of CSS properties for the
	 * text. Equivalent attributes for the hover state are given in
	 * <code>theme.states.hover</code>.
	 * @param <K> type for the key
	 * <p>
	 * if style was set using {@link #setTheme(Map)}, type will be the same as the type from the key of the map
	 * <p>
	 * if style was not set, type will be Object by default
	 * @param <V> type for the value
	 * <p>
	 * if style was set using {@link #setTheme(Map)}, type will be the same as the type from the key of the map
	 * <p>
	 * if style was not set, type will be Object by default
	 */
	public <K, V> Map<K, V> getTheme() {
		return Generics.cast(getAttr(Attrs.theme, null).asValue());
	}

	/**
	 * Sets a collection of attributes for the button. The object takes SVG
	 * attributes like <code>fill</code>, <code>stroke</code>,
	 * <code>stroke-width</code> or <code>r</code>, the border radius. The theme
	 * also supports <code>style</code>, a collection of CSS properties for the
	 * text. Equivalent attributes for the hover state are given in
	 * <code>theme.states.hover</code>.<p>
	 * This method allows <b>complex</b> styling like:
	 * <pre>
	 * {@code
	 * Map<String, Object> styles = new HashMap<String, Object>();
	 * styles.put("fill", "#E8E8EA");
	 * styles.put("stroke", "#6E6E78");
	 * styles.put("stroke-width", "1");
	 * styles.put("r", "0");
	 * Map<String, Object> states = new HashMap<String, Object>();
	 * Map<String, String> hover = new HashMap<String, String>();
	 * hover.put("fill", "#F4F4F6");
	 * hover.put("stroke", "#6E6E78");
	 * hover.put("stroke-width", "1");
	 * hover.put("r", "0");
	 * states.put("hover", hover);
	 * styles.put("states", states);
	 * Map<String, String> style = new HashMap<String, String>();
	 * style.put("color", "#4D5663");
	 * style.put("fontSize", "12px");
	 * style.put("fontFamily", "'Arial'");
	 * style.put("fontWeight", "normal");
	 * style.put("padding", "10px");
	 * styles.put("style", style);
	 * }</pre>
	 * which will generate the final results like:
	 * <pre>{@code
	 * {
	 *     "fill": "#E8E8EA",
	 *     "stroke": "#6E6E78",
	 *     "stroke-width": "1",
	 *     "r": "0",
	 *     "states":
	 *     {
	 *         "hover":
	 *         {
	 *             "fill": "#F4F4F6",
	 *             "stroke": "#6E6E78",
	 *             "stroke-width": "1",
	 *             "r": "0"
	 *         }
	 *     },
	 *     "style":
	 *     {
	 *         "color": "#4D5663",
	 *         "fontSize": "12px",
	 *         "fontFamily": "'Arial'",
	 *         "fontWeight": "normal",
	 *         "padding": "10px"
	 *     }
	 * }
	 * }</pre>
	 * @param <K> Type for the key, usually String
	 * @param <V> Type for the value, usually String. For nested maps, use Object or any other type.
	 * @param theme the theme object
	 */
	public <K, V> void setTheme(Map<K, V> theme) {
		setAttr(Attrs.theme, theme);
	}
}