/* Navigation.java

	Purpose:
		
	Description:
		
	History:
		Fri, Jan 17, 2014 12:36:59 PM, Created by RaymondChao

Copyright (C) 2014 Potix Corporation. All Rights Reserved.

 */
package org.zkoss.chart;

import java.util.LinkedHashMap;
import java.util.Map;

import org.zkoss.chart.util.DynamicalAttribute;
import org.zkoss.chart.util.MapsHelper;
import org.zkoss.lang.Generics;

/**
 * A collection of options for buttons and menus appearing in the exporting
 * module.
 * <p>
 * All the options in this class support {@link DynamicalAttribute}.
 * 
 * @author RaymondChao
 */
public class Navigation extends Optionable {
	private enum Attrs implements PlotAttribute, DynamicalAttribute {
		annotationsOptions,
		buttonOptions, menuItemHoverStyle, menuItemStyle, menuStyle
	}

	/**
	 * Additional options to be merged into all annotations, if null, create a
	 * new one.
	 * @since 10.2.1.0
	 */
	public Annotation getAnnotationsOptions() {
		return (Annotation) getAttr(Attrs.annotationsOptions,
				Annotation.class).asValue();
	}


	/**
	 * Additional options to be merged into all annotations.
	 * @since 10.2.1.0
	 */
	public void setAnnotationsOptions(Annotation annotationsOptions) {
		setAttr(Attrs.annotationsOptions, annotationsOptions);
	}


	/**
	 * Returns a collection of options for buttons appearing in the exporting
	 * module.
	 */
	public NavigationButton getButtonOptions() {
		 NavigationButton options = (NavigationButton) getAttr(Attrs.buttonOptions);
		 if (options == null) {
			 options = new NavigationButton();
			 setButtonOptions(options);
		 }
		 return options;
	}

	/**
	 * Sets a collection of options for buttons appearing in the exporting
	 * module.
	 */
	public void setButtonOptions(NavigationButton buttonOptions) {
		setAttr(Attrs.buttonOptions, buttonOptions);
	}

	/**
	 * Returns CSS styles for the hover state of the individual items within the
	 * popup menu appearing by default when the export icon is clicked. The menu
	 * items are rendered in HTML.
	 * <p>
	 * Defaults to
	 * 
	 * <pre>
	 * background: #4572A5;	color: #FFFFFF;
	 * </pre>
	 * @param <K> type for the key
	 * <p>
	 * if style was set using {@link #setMenuItemHoverStyle(String)}, type will be String
	 * <p>
	 * if style was set using {@link #setMenuItemHoverStyle(Map)}, type will be the same as the type from the key of the map
	 * <p>
	 * if style was not set, default style will be applied, and type will be String
	 * @param <V> type for the value
	 * <p>
	 * if style was set using {@link #setMenuItemHoverStyle(String)}, type will be String
	 * <p>
	 * if style was set using {@link #setMenuItemHoverStyle(Map)}, type will be the same as the type from the key of the map
	 * <p>
	 * if style was not set, default style will be applied, and type will be String
	 */
	public <K, V> Map<K, V> getMenuItemHoverStyle() {
		if (!containsKey(Attrs.menuItemHoverStyle)) {
			setMenuItemHoverStyle("background: #4572A5;	color: #FFFFFF;");
		}
		return Generics.cast(getAttr(Attrs.menuItemHoverStyle));
	}

	/**
	 * Sets CSS styles for the hover state of the individual items within the
	 * popup menu appearing by default when the export icon is clicked. The menu
	 * items are rendered in HTML.
	 * <p>
	 * This method allows <b>simple</b> settings like
	 * <pre>{@code
	 * label.setStyle("fontWeight: 'bold'; color: 'white'; textShadow: '0px 1px 2px black'");
	 * }</pre>
	 * <p>
	 * In the format of CSS key:value, and ";" separating multiple key:value pairs,
	 * if you have more a complex CSS styling, we recommend using {@link #setMenuItemHoverStyle(Map)}
	 * @param menuItemHoverStyle one or many CSS key:value pairs in a single line string format
	 */
	public void setMenuItemHoverStyle(String menuItemHoverStyle) {
		setMenuItemHoverStyle(MapsHelper.parse(new LinkedHashMap<String, String>(), menuItemHoverStyle, ':', ';',
				'\''));
	}

	/**
	 * Sets CSS styles for the hover state of the individual items within the
	 * popup menu appearing by default when the export icon is clicked. The menu
	 * items are rendered in HTML.
	 * <p>
	 * This method allows <b>complex</b> styling like:
	 * <pre>
	 * {@code
	 * Map<String, Object> styles = new HashMap<String, Object>();
	 * styles.put("fill", "#E8E8EA");
	 * styles.put("stroke", "#6E6E78");
	 * styles.put("stroke-width", "1");
	 * styles.put("r", "0");
	 * Map<String, Object> states = new HashMap<String, Object>();
	 * Map<String, String> hover = new HashMap<String, String>();
	 * hover.put("fill", "#F4F4F6");
	 * hover.put("stroke", "#6E6E78");
	 * hover.put("stroke-width", "1");
	 * hover.put("r", "0");
	 * states.put("hover", hover);
	 * styles.put("states", states);
	 * Map<String, String> style = new HashMap<String, String>();
	 * style.put("color", "#4D5663");
	 * style.put("fontSize", "12px");
	 * style.put("fontFamily", "'Arial'");
	 * style.put("fontWeight", "normal");
	 * style.put("padding", "10px");
	 * styles.put("style", style);
	 * }</pre>
	 * which will generate the final results like:
	 * <pre>{@code
	 * {
	 *     "fill": "#E8E8EA",
	 *     "stroke": "#6E6E78",
	 *     "stroke-width": "1",
	 *     "r": "0",
	 *     "states":
	 *     {
	 *         "hover":
	 *         {
	 *             "fill": "#F4F4F6",
	 *             "stroke": "#6E6E78",
	 *             "stroke-width": "1",
	 *             "r": "0"
	 *         }
	 *     },
	 *     "style":
	 *     {
	 *         "color": "#4D5663",
	 *         "fontSize": "12px",
	 *         "fontFamily": "'Arial'",
	 *         "fontWeight": "normal",
	 *         "padding": "10px"
	 *     }
	 * }
	 * }</pre>
	 * @param <K> Type for the key, usually String
	 * @param <V> Type for the value, usually String. For nested maps, use Object or any other type.
	 * @param menuItemHoverStyle the style object
	 */
	public <K, V> void setMenuItemHoverStyle(Map<K, V> menuItemHoverStyle) {
		setAttr(Attrs.menuItemHoverStyle, menuItemHoverStyle, NOT_NULL_VALUE);
	}

	/**
	 * Returns CSS styles for the individual items within the popup menu
	 * appearing by default when the export icon is clicked. The menu items are
	 * rendered in HTML.
	 * <p>
	 * Defaults to
	 * 
	 * <pre>
	 * padding: 0 5px; background: NONE; color: #303030;
	 * </pre>
	 * @param <K> type for the key
	 * <p>
	 * if style was set using {@link #setMenuItemStyle(String)}, type will be String
	 * <p>
	 * if style was set using {@link #setMenuItemStyle(Map)}, type will be the same as the type from the key of the map
	 * <p>
	 * if style was not set, default style will be applied, and type will be String
	 * @param <V> type for the value
	 * <p>
	 * if style was set using {@link #setMenuItemStyle(String)}, type will be String
	 * <p>
	 * if style was set using {@link #setMenuItemStyle(Map)}, type will be the same as the type from the key of the map
	 * <p>
	 * if style was not set, default style will be applied, and type will be String
	 */
	public <K, V> Map<K, V> getMenuItemStyle() {
		if (!containsKey(Attrs.menuItemStyle)) {
			setMenuItemStyle("padding: 0 5px; background: NONE; color: #303030;");
		}
		return Generics.cast(getAttr(Attrs.menuItemStyle));
	}

	/**
	 * Sets CSS styles for the individual items within the popup menu appearing
	 * by default when the export icon is clicked. The menu items are rendered
	 * in HTML.
	 * <p>
	 * This method allows <b>simple</b> settings like
	 * <pre>{@code
	 * label.setStyle("fontWeight: 'bold'; color: 'white'; textShadow: '0px 1px 2px black'");
	 * }</pre>
	 * <p>
	 * In the format of CSS key:value, and ";" separating multiple key:value pairs,
	 * if you have more a complex CSS styling, we recommend using {@link #setMenuItemStyle(Map)}
	 * @param menuItemStyle one or many CSS key:value pairs in a single line string format
	 */
	public void setMenuItemStyle(String menuItemStyle) {
		setMenuItemStyle(MapsHelper.parse(new LinkedHashMap<String, String>(), menuItemStyle, ':', ';', '\''));
	}

	/**
	 * Sets CSS styles for the individual items within the popup menu appearing
	 * by default when the export icon is clicked. The menu items are rendered
	 * in HTML.
	 * <p>
	 * This method allows <b>complex</b> styling like:
	 * <pre>
	 * {@code
	 * Map<String, Object> styles = new HashMap<String, Object>();
	 * styles.put("fill", "#E8E8EA");
	 * styles.put("stroke", "#6E6E78");
	 * styles.put("stroke-width", "1");
	 * styles.put("r", "0");
	 * Map<String, Object> states = new HashMap<String, Object>();
	 * Map<String, String> hover = new HashMap<String, String>();
	 * hover.put("fill", "#F4F4F6");
	 * hover.put("stroke", "#6E6E78");
	 * hover.put("stroke-width", "1");
	 * hover.put("r", "0");
	 * states.put("hover", hover);
	 * styles.put("states", states);
	 * Map<String, String> style = new HashMap<String, String>();
	 * style.put("color", "#4D5663");
	 * style.put("fontSize", "12px");
	 * style.put("fontFamily", "'Arial'");
	 * style.put("fontWeight", "normal");
	 * style.put("padding", "10px");
	 * styles.put("style", style);
	 * }</pre>
	 * which will generate the final results like:
	 * <pre>{@code
	 * {
	 *     "fill": "#E8E8EA",
	 *     "stroke": "#6E6E78",
	 *     "stroke-width": "1",
	 *     "r": "0",
	 *     "states":
	 *     {
	 *         "hover":
	 *         {
	 *             "fill": "#F4F4F6",
	 *             "stroke": "#6E6E78",
	 *             "stroke-width": "1",
	 *             "r": "0"
	 *         }
	 *     },
	 *     "style":
	 *     {
	 *         "color": "#4D5663",
	 *         "fontSize": "12px",
	 *         "fontFamily": "'Arial'",
	 *         "fontWeight": "normal",
	 *         "padding": "10px"
	 *     }
	 * }
	 * }</pre>
	 * @param <K> Type for the key, usually String
	 * @param <V> Type for the value, usually String. For nested maps, use Object or any other type.
	 * @param menuItemStyle the style object
	 */
	public <K, V> void setMenuItemStyle(Map<K, V> menuItemStyle) {
		setAttr(Attrs.menuItemStyle, menuItemStyle, NOT_NULL_VALUE);
	}

	/**
	 * Returns CSS styles for the popup menu appearing by default when the
	 * export icon is clicked. This menu is rendered in HTML.
	 * <p>Defaults to
	 * 
	 * <pre>
	 * border: 1px solid #A0A0A0; background: #FFFFFF;
	 * </pre>
	 * @param <K> type for the key
	 * <p>
	 * if style was set using {@link #setMenuStyle(String)}, type will be String
	 * <p>
	 * if style was set using {@link #setMenuStyle(Map)}, type will be the same as the type from the key of the map
	 * <p>
	 * if style was not set, default style will be applied, and type will be String
	 * @param <V> type for the value
	 * <p>
	 * if style was set using {@link #setMenuStyle(String)}, type will be String
	 * <p>
	 * if style was set using {@link #setMenuStyle(Map)}, type will be the same as the type from the key of the map
	 * <p>
	 * if style was not set, default style will be applied, and type will be String
	 */
	public <K, V> Map<K, V> getMenuStyle() {
		if (!containsKey(Attrs.menuStyle)) {
			setMenuStyle("border: 1px solid #A0A0A0; background: #FFFFFF;");
		}
		return Generics.cast(getAttr(Attrs.menuStyle));
	}

	/**
	 * Sets CSS styles for the popup menu appearing by default when the
	 * export icon is clicked. This menu is rendered in HTML.
	 * <p>
	 * This method allows <b>simple</b> settings like
	 * <pre>{@code
	 * label.setStyle("fontWeight: 'bold'; color: 'white'; textShadow: '0px 1px 2px black'");
	 * }</pre>
	 * <p>
	 * In the format of CSS key:value, and ";" separating multiple key:value pairs,
	 * if you have more a complex CSS styling, we recommend using {@link #setMenuStyle(Map)}
	 * @param menuStyle one or many CSS key:value pairs in a single line string format
	 */
	public void setMenuStyle(String menuStyle) {
		setMenuStyle(MapsHelper.parse(new LinkedHashMap<String, String>(), menuStyle, ':', ';', '\''));
	}

	/**
	 * Sets CSS styles for the popup menu appearing by default when the
	 * export icon is clicked. This menu is rendered in HTML.
	 * <p>
	 * This method allows <b>complex</b> styling like:
	 * <pre>
	 * {@code
	 * Map<String, Object> styles = new HashMap<String, Object>();
	 * styles.put("fill", "#E8E8EA");
	 * styles.put("stroke", "#6E6E78");
	 * styles.put("stroke-width", "1");
	 * styles.put("r", "0");
	 * Map<String, Object> states = new HashMap<String, Object>();
	 * Map<String, String> hover = new HashMap<String, String>();
	 * hover.put("fill", "#F4F4F6");
	 * hover.put("stroke", "#6E6E78");
	 * hover.put("stroke-width", "1");
	 * hover.put("r", "0");
	 * states.put("hover", hover);
	 * styles.put("states", states);
	 * Map<String, String> style = new HashMap<String, String>();
	 * style.put("color", "#4D5663");
	 * style.put("fontSize", "12px");
	 * style.put("fontFamily", "'Arial'");
	 * style.put("fontWeight", "normal");
	 * style.put("padding", "10px");
	 * styles.put("style", style);
	 * }</pre>
	 * which will generate the final results like:
	 * <pre>{@code
	 * {
	 *     "fill": "#E8E8EA",
	 *     "stroke": "#6E6E78",
	 *     "stroke-width": "1",
	 *     "r": "0",
	 *     "states":
	 *     {
	 *         "hover":
	 *         {
	 *             "fill": "#F4F4F6",
	 *             "stroke": "#6E6E78",
	 *             "stroke-width": "1",
	 *             "r": "0"
	 *         }
	 *     },
	 *     "style":
	 *     {
	 *         "color": "#4D5663",
	 *         "fontSize": "12px",
	 *         "fontFamily": "'Arial'",
	 *         "fontWeight": "normal",
	 *         "padding": "10px"
	 *     }
	 * }
	 * }</pre>
	 * @param <K> Type for the key, usually String
	 * @param <V> Type for the value, usually String. For nested maps, use Object or any other type.
	 * @param menuStyle the style object
	 */
	public <K, V> void setMenuStyle(Map<K, V> menuStyle) {
		setAttr(Attrs.menuStyle, menuStyle, NOT_NULL_VALUE);
	}

}