/* CollapseButton.java

	Purpose:
		
	Description:
		
	History:
		10:57 AM 2023/4/7, Created by jumperchen

Copyright (C) 2023 Potix Corporation. All Rights Reserved.
*/
package org.zkoss.chart.plotOptions;

import java.util.LinkedHashMap;
import java.util.Map;

import org.zkoss.chart.Optionable;
import org.zkoss.chart.PlotAttribute;
import org.zkoss.chart.util.DynamicalAttribute;
import org.zkoss.chart.util.MapsHelper;
import org.zkoss.lang.Generics;

/**
 * Options applied to collapse Button.
 * The collapse button is the small button which indicates,
 * that the node is collapsable.
 * @author jumperchen
 * @since 10.3.3.0
 */
public class CollapseButton extends Optionable {
	private enum Attrs implements PlotAttribute, DynamicalAttribute {
		enabled,
		height,
		onlyOnHover,
		shape,
		style,
		width,
		x,
		y
	}

	/**
	 * Returns whether the button should be visible.
	 * <p>
	 * Default: true
	 */
	public boolean isEnabled() {
		return getAttr(Attrs.enabled, true).asBoolean();
	}

	/**
	 * Sets whether the button should be visible.
	 * <p>
	 * Default: true
	 */
	public void setEnabled(boolean enabled) {
		setAttr(Attrs.enabled, enabled);
	}

	/**
	 * Returns pixel height of the button.
	 * <p>
	 * Default: 10
	 */
	public Number getHeight() {
		return getAttr(Attrs.height, 10).asNumber();
	}

	/**
	 * Sets pixel height of the button.
	 * <p>
	 * Default: 10
	 */
	public void setHeight(Number height) {
		setAttr(Attrs.height, height);
	}

	/**
	 * Returns whether the button should be visible only when the node is hovered.
	 * When set to true, the button is hidden for nodes, which are not collapsed,
	 * and shown for the collapsed ones.
	 * <p>
	 * Default: true
	 */
	public boolean isOnlyOnHover() {
		return getAttr(Attrs.onlyOnHover, true).asBoolean();
	}

	/**
	 * Sets whether the button should be visible only when the node is hovered.
	 * When set to true, the button is hidden for nodes, which are not collapsed,
	 * and shown for the collapsed ones.
	 * <p>
	 * Default: true
	 */
	public void setOnlyOnHover(boolean onlyOnHover) {
		setAttr(Attrs.onlyOnHover, onlyOnHover);
	}

	/**
	 * Returns pixel width of the button.
	 * <p>
	 * Default: 10
	 */
	public Number getWidth() {
		return getAttr(Attrs.width, 10).asNumber();
	}

	/**
	 * Sets pixel width of the button.
	 * <p>
	 * Default: 10
	 */
	public void setWidth(Number width) {
		setAttr(Attrs.width, width);
	}

	/**
	 * Returns the x position offset of the button.
	 * <p>
	 * Default: 0
	 */
	public Number getX() {
		return getAttr(Attrs.x, 0).asNumber();
	}


	/**
	 * Sets the x position offset of the button.
	 * <p>
	 * Default: 0
	 */
	public void setX(Number x) {
		setAttr(Attrs.x, x);
	}


	/**
	 * Returns the y position offset of the button.
	 * <p>
	 * Default: 0
	 */
	public Number getY() {
		return getAttr(Attrs.y, 0).asNumber();
	}

	/**
	 * Sets the y position offset of the button.
	 * <p>
	 * Default: 0
	 */
	public void setY(Number y) {
		setAttr(Attrs.y, y);
	}


	/**
	 * Returns CSS styles for the annotation's label.
	 * <p>
	 * Default: empty
	 * @param <K> type for the key
	 * <p>
	 * if style was set using {@link #setStyle(String)}, type will be String
	 * <p>
	 * if style was set using {@link #setStyle(Map)}, type will be the same as the type from the key of the map
	 * <p>
	 * if style was not set, default style will be applied, and type will be String
	 * @param <V> type for the value
	 * <p>
	 * if style was set using {@link #setStyle(String)}, type will be String
	 * <p>
	 * if style was set using {@link #setStyle(Map)}, type will be the same as the type from the key of the map
	 * <p>
	 * if style was not set, default style will be applied, and type will be String
	 */
	public <K, V> Map<K, V> getStyle() {
		if (!containsKey(Attrs.style)) {
			setStyle("");
		}
		return Generics.cast(getAttr(Attrs.style));
	}

	/**
	 * Sets CSS styles for the annotation's label.
	 * <p>
	 * This method allows <b>simple</b> settings like
	 * <pre>{@code
	 * label.setStyle("fontWeight: 'bold'; color: 'white'; textShadow: '0px 1px 2px black'");
	 * }</pre>
	 * <p>
	 * In the format of CSS key:value, and ";" separating multiple key:value pairs,
	 * if you have more a complex CSS styling, we recommend using {@link #setStyle(Map)}
	 * @param style one or many CSS key:value pairs in a single line string format
	 */
	public void setStyle(String style) {
		setStyle(
				MapsHelper.parse(new LinkedHashMap<String, String>(), style, ':', ';', '\''));
	}

	/**
	 * Sets CSS styles for the annotation's label.
	 * <p>
	 * This method allows <b>complex</b> styling like:
	 * <pre>
	 * {@code
	 * Map<String, Object> styles = new HashMap<String, Object>();
	 * styles.put("fill", "#E8E8EA");
	 * styles.put("stroke", "#6E6E78");
	 * styles.put("stroke-width", "1");
	 * styles.put("r", "0");
	 * Map<String, Object> states = new HashMap<String, Object>();
	 * Map<String, String> hover = new HashMap<String, String>();
	 * hover.put("fill", "#F4F4F6");
	 * hover.put("stroke", "#6E6E78");
	 * hover.put("stroke-width", "1");
	 * hover.put("r", "0");
	 * states.put("hover", hover);
	 * styles.put("states", states);
	 * Map<String, String> style = new HashMap<String, String>();
	 * style.put("color", "#4D5663");
	 * style.put("fontSize", "12px");
	 * style.put("fontFamily", "'Arial'");
	 * style.put("fontWeight", "normal");
	 * style.put("padding", "10px");
	 * styles.put("style", style);
	 * }</pre>
	 * which will generate the final results like:
	 * <pre>{@code
	 * {
	 *     "fill": "#E8E8EA",
	 *     "stroke": "#6E6E78",
	 *     "stroke-width": "1",
	 *     "r": "0",
	 *     "states":
	 *     {
	 *         "hover":
	 *         {
	 *             "fill": "#F4F4F6",
	 *             "stroke": "#6E6E78",
	 *             "stroke-width": "1",
	 *             "r": "0"
	 *         }
	 *     },
	 *     "style":
	 *     {
	 *         "color": "#4D5663",
	 *         "fontSize": "12px",
	 *         "fontFamily": "'Arial'",
	 *         "fontWeight": "normal",
	 *         "padding": "10px"
	 *     }
	 * }
	 * }</pre>
	 * @param <K> Type for the key, usually String
	 * @param <V> Type for the value, usually String. For nested maps, use Object or any other type.
	 * @param style the style object
	 */
	public <K, V> void setStyle(Map<K, V> style) {
		setAttr(Attrs.style, style, NOT_NULL_VALUE);
	}
}
