/* ColumnRangePlotOptions.java

	Purpose:
		
	Description:
		
	History:
		Thu, Jan 16, 2014  3:58:08 PM, Created by RaymondChao

Copyright (C) 2014 Potix Corporation. All Rights Reserved.

 */
package org.zkoss.chart.plotOptions;

import java.util.Arrays;
import java.util.List;
import java.util.stream.Collectors;

import org.zkoss.chart.Charts;
import org.zkoss.chart.Color;
import org.zkoss.chart.LinearGradient;
import org.zkoss.chart.PlotAttribute;
import org.zkoss.chart.RadialGradient;
import org.zkoss.chart.util.DynamicalAttribute;
import org.zkoss.lang.Generics;

/**
 * An option for column range plot
 * 
 * <p>
 * All the options in this class support {@link DynamicalAttribute}.
 * @author RaymondChao
 */
public class ColumnRangePlotOptions extends SeriesPlotOptions {
	private enum Attrs implements PlotAttribute, DynamicalAttribute {
		borderColor,
		borderRadius,
		centerInCategory,
		colors,
		depth,
		edgeColor,
		edgeWidth,
		grouping,
		groupPadding,
		maxPointWidth,
		minPointLength,
		pointPadding,
		pointRange,
		pointWidth
	}


	/**
	 * Returns the color of the border surrounding each slice.
	 * <p>
	 * Default: null
	 * @since 10.2.1.0
	 */
	public Color getBorderColor() {
		return (Color) getAttr(Attrs.borderColor, null).asValue();
	}

	/**
	 * Sets the color of the border surrounding each slice.
	 * @since 10.2.1.0
	 */
	public void setBorderColor(Color borderColor) {
		setAttr(Attrs.borderColor, borderColor);
	}

	/**
	 * Sets the color of the border surrounding each slice.
	 * @since 10.2.1.0
	 */
	public void setBorderColor(String borderColor) {
		setBorderColor(new Color(borderColor));
	}

	/**
	 * Sets the color of the border surrounding each slice.
	 * @since 10.2.1.0
	 */
	public void setBorderColor(LinearGradient borderColor) {
		setBorderColor(new Color(borderColor));
	}

	/**
	 * Sets the color of the border surrounding each slice.
	 * @since 10.2.1.0
	 */
	public void setBorderColor(RadialGradient borderColor) {
		setBorderColor(new Color(borderColor));
	}

	/**
	 * Returns the border radius of the series.
	 * @since 10.2.1.0
	 */
	public Number getBorderRadius() {
		return getAttr(Attrs.borderRadius, null).asNumber();
	}

	/**
	 * Sets the border radius of the series.
	 * @since 10.2.1.0
	 */
	public void setBorderRadius(Number borderRadius) {
		setAttr(Attrs.borderRadius, borderRadius);
	}

	/**
	 * When true, the columns will center in the category, ignoring null or missing points.
	 * When false, space will be reserved for null or missing points.
	 *
	 * <p>
	 * Default: {@code false}.
	 * @since 10.2.1.0
	 */
	public boolean isCenterInCategory() {
		return getAttr(Attrs.centerInCategory, false).asBoolean();
	}

	/**
	 * When true, the columns will center in the category, ignoring null or missing points.
	 * When false, space will be reserved for null or missing points.
	 * @since 10.2.1.0
	 */
	public void setCenterInCategory(boolean centerInCategory) {
		setAttr(Attrs.centerInCategory, centerInCategory);
	}



	/**
	 * Returns a series specific or series type specific color set to apply
	 * instead of the global {@link Charts#getColors()} when
	 * {@link #setColorByPoint(boolean)} is true.
	 * @since 10.2.1.0
	 */
	public List<Color> getColors() {
		return Generics.cast(getAttr(Attrs.colors, null).asValue());
	}

	/**
	 * Sets a series specific or series type specific color set to apply
	 * instead of the global {@link Charts#getColors()} when
	 * {@link #setColorByPoint(boolean)} is true.
	 * @since 10.2.1.0
	 */
	public void setColors(List<Color> colors) {
		setAttr(Attrs.colors, colors);
	}

	/**
	 * Sets a series specific or series type specific color set to apply
	 * instead of the global {@link Charts#getColors()} when
	 * {@link #setColorByPoint(boolean)} is true.
	 * @since 10.2.1.0
	 */
	public void setColors(String... colors) {
		setColors(Arrays.stream(colors).map(Color::new)
				.collect(Collectors.toList()));
	}

	/**
	 * Sets a series specific or series type specific color set to apply
	 * instead of the global {@link Charts#getColors()} when
	 * {@link #setColorByPoint(boolean)} is true.
	 * @since 10.2.1.0
	 */
	public void setColors(Color... colors) {
		setColors(Arrays.asList(colors));
	}

	@Override
	protected DataLabels newDataLabels() {
		return new RangeDataLabels();
	}

	@Override
	public RangeDataLabels getDataLabels() {
		return (RangeDataLabels) super.getDataLabels();
	}

	/**
	 * Returns the depth of the columns in a 3D column range chart.
	 * <p>
	 * Default: 25
	 */
	public Number getDepth() {
		return getAttr(Attrs.depth, 25).asNumber();
	}

	/**
	 * Sets the depth of the columns in a 3D column range chart.
	 */
	public void setDepth(Number depth) {
		setAttr(Attrs.depth, depth);
	}

	/**
	 * 3D columns only. The color of the edges. Similar to {@code borderColor},
	 * except it defaults to the same color as the column.
	 * <p>Default: {@code null}
	 * @since 10.2.1.0
	 */
	public Color getEdgeColor() {
		return (Color) getAttr(Attrs.edgeColor, null).asValue();
	}

	/**
	 * 3D columns only. The color of the edges. Similar to {@code borderColor},
	 * except it defaults to the same color as the column.
	 * <p>Default: {@code null}
	 * @since 10.2.1.0
	 */
	public void setEdgeColor(Color color) {
		setAttr(Attrs.edgeColor, color);
	}

	/**
	 * 3D columns only. The color of the edges. Similar to {@code borderColor},
	 * except it defaults to the same color as the column.
	 * <p>Default: {@code null}
	 * @since 10.2.1.0
	 */
	public void setEdgeColor(String color) {
		setColor(new Color(color));
	}

	/**
	 * 3D columns only.
	 * Returns the width of the colored edges.
	 * <p>
	 * Default: 1
	 * @since 10.2.1.0
	 */
	public Number getEdgeWidth() {
		return getAttr(Attrs.edgeWidth, 1).asNumber();
	}

	/**
	 * 3D columns only.
	 * Sets the width of the colored edges.
	 * @since 10.2.1.0
	 */
	public void setEdgeWidth(Number edgeWidth) {
		setAttr(Attrs.edgeWidth, edgeWidth);
	}

	/**
	 * Returns whether to group non-stacked columns or to let them render independent
	 * of each other. Non-grouped columns will be laid out individually and overlap each other.
	 * <p>Default: {@code true}
	 * @since 10.2.1.0
	 */
	public boolean isGrouping() {
		return getAttr(Attrs.grouping, true).asBoolean();
	}

	/**
	 * Sets whether to group non-stacked columns or to let them render independent
	 * of each other. Non-grouped columns will be laid out individually and overlap each other.
	 * <p>Default: {@code true}
	 * @since 10.2.1.0
	 */
	public void setGrouping(boolean grouping) {
		setAttr(Attrs.grouping, grouping);
	}

	/**
	 * Returns padding between each value groups, in x axis units.
	 * <p>Default: 0.2
	 * @since 10.2.1.0
	 */
	public Number getGroupPadding() {
		return getAttr(Attrs.groupPadding, 0.2).asNumber();
	}

	/**
	 * Sets padding between each value groups, in x axis units.
	 * @since 10.2.1.0
	 */
	public void setGroupPadding(Number groupPadding) {
		setAttr(Attrs.groupPadding, groupPadding);
	}

	/**
	 * Returns the maximum allowed pixel width for a column,
	 * translated to the height of a bar in a bar chart.
	 * This prevents the columns from becoming too wide when there is a small
	 * number of points in the chart.
	 * <p>Default: null
	 * @since 10.2.1.0
	 */
	public Number getMaxPointWidth() {
		return getAttr(Attrs.maxPointWidth, null).asNumber();
	}

	/**
	 * Sets the maximum allowed pixel width for a column,
	 * translated to the height of a bar in a bar chart.
	 * This prevents the columns from becoming too wide when there is a small
	 * number of points in the chart.
	 * @since 10.2.1.0
	 */
	public void setMaxPointWidth(Number maxPointWidth) {
		setAttr(Attrs.maxPointWidth, maxPointWidth);
	}

	/**
	 * Returns the minimal height for a column or width for a bar. By default, 0
	 * values are not shown. To visualize a 0 (or close to zero) point, set the
	 * minimal point length to a pixel value like 3. In stacked column charts,
	 * minPointLength might not be respected for tightly packed values.
	 * <p>Default: 0
	 * @since 10.2.1.0
	 */
	public Number getMinPointLength() {
		return getAttr(Attrs.minPointLength, 0).asNumber();
	}

	/**
	 * Sets the minimal height for a column or width for a bar. By default, 0
	 * values are not shown. To visualize a 0 (or close to zero) point, set the
	 * minimal point length to a pixel value like 3. In stacked column charts,
	 * minPointLength might not be respected for tightly packed values.
	 * @since 10.2.1.0
	 */
	public void setMinPointLength(Number minPointLength) {
		setAttr(Attrs.minPointLength, minPointLength, 0);
	}

	/**
	 * Returns padding between each column or bar, in x axis units.
	 * <p> Default: {@code 0.1}
	 * @since 10.2.1.0
	 */
	public Number getPointPadding() {
		return getAttr(Attrs.pointPadding, 0.1).asNumber();
	}

	/**
	 * Sets padding between each column or bar, in x axis units. 'pointPadding
	 * padding between each column or bar, in x axis units.
	 * @since 10.2.1.0
	 */
	public void setPointPadding(Number pointPadding) {
		setAttr(Attrs.pointPadding, pointPadding);
	}

	/**
	 * Returns the X axis range that each point is valid for. This determines
	 * the width of the column. On a categorized axis, the range will be 1 by
	 * default (one category unit). On linear and datetime axes, the range will
	 * be computed as the distance between the two closest data points.
	 * <p> Default: null
	 * @since 10.2.1.0
	 */
	public Number getPointRange() {
		return getAttr(Attrs.pointRange, null).asNumber();
	}

	/**
	 * Sets the X axis range that each point is valid for. This determines the
	 * width of the column. On a categorized axis, the range will be 1 by
	 * default (one category unit). On linear and datetime axes, the range will
	 * be computed as the distance between the two closest data points.
	 * @since 10.2.1.0
	 */
	public void setPointRange(Number pointRange) {
		setAttr(Attrs.pointRange, pointRange);
	}

	/**
	 * Returns a pixel value specifying a fixed width for each column or bar.
	 * When <code>null</code>, the width is calculated from the
	 * <code>pointPadding</code> and <code>groupPadding</code>.
	 * <p>Default: null
	 * @since 10.2.1.0
	 */
	public Number getPointWidth() {
		return getAttr(Attrs.pointWidth, null).asNumber();
	}

	/**
	 * Sets a pixel value specifying a fixed width for each column or bar. When
	 * <code>null</code>, the width is calculated from the
	 * <code>pointPadding</code> and <code>groupPadding</code>.
	 * @since 10.2.1.0
	 */
	public void setPointWidth(Number pointWidth) {
		setAttr(Attrs.pointWidth, pointWidth);
	}
}