/* HistogramPlotOptions.java

	Purpose:
		
	Description:
		
	History:
		Thu Nov 21 11:17:27 CST 2019, Created by rudyhuang

Copyright (C) 2019 Potix Corporation. All Rights Reserved.
*/
package org.zkoss.chart.plotOptions;

import java.util.Arrays;
import java.util.List;
import java.util.stream.Collectors;

import org.zkoss.chart.Color;
import org.zkoss.chart.PlotAttribute;
import org.zkoss.json.JavaScriptValue;
import org.zkoss.lang.Generics;

/**
 * An option for histogram plot
 *
 * @author rudyhuang
 * @since 7.2.1.0
 */
public class HistogramPlotOptions extends SeriesPlotOptions {
	private enum Attrs implements PlotAttribute {
		binsNumber, binWidth, borderColor, borderRadius, centerInCategory,
		colors, depth, edgeColor, edgeWidth, grouping, groupPadding,
		groupZPadding, maxPointWidth, minPointLength, pointPadding, pointRange,
		pointWidth
	}

	/**
	 * Sets the series specific or series type specific color set to apply
	 * instead of the global colors when {@link #isColorByPoint()} is true.
	 * @since 10.2.1.0
	 */
	public void setColors(List<Color> colors) {
		setAttr(Attrs.colors, colors);
	}

	/**
	 * Sets the series specific or series type specific color set to apply
	 * instead of the global colors when {@link #isColorByPoint()} is true.
	 * @since 10.2.1.0
	 */
	public void setColors(String... colors) {
		setColors(Arrays.stream(colors).map(Color::new).collect(Collectors.toList()));
	}

	/**
	 * Sets the series specific or series type specific color set to apply
	 * instead of the global colors when {@link #isColorByPoint()} is true.
	 * @since 10.2.1.0
	 */
	public void setColors(Color... colors) {
		setColors(Arrays.asList(colors));
	}

	/**
	 * Returns the series specific or series type specific color set to apply
	 * instead of the global colors when {@link #isColorByPoint()} is true.
	 * @since 10.2.1.0
	 */
	public List<Color> getColors() {
		return Generics.cast(getAttr(Attrs.colors, null).asValue());
	}

	/**
	 * Returns the depth of the 3D chart.
	 * <p>Default: {@code 25}
	 * @since 10.2.1.0
	 */
	public Number getDepth() {
		return getAttr(Attrs.depth, 25).asNumber();
	}

	/**
	 * Sets the depth of the 3D chart.
	 * <p>Default: {@code null}
	 * @since 10.2.1.0
	 */
	public void setDepth(Number depth) {
		setAttr(Attrs.depth, depth);
	}

	/**
	 * When true, the columns will center in the category, ignoring null or missing points.
	 * When false, space will be reserved for null or missing points.
	 *
	 * <p>
	 * Default: {@code false}.
	 * @since 10.2.1.0
	 */
	public boolean isCenterInCategory() {
		return getAttr(Attrs.centerInCategory, false).asBoolean();
	}

	/**
	 * When true, the columns will center in the category, ignoring null or missing points.
	 * When false, space will be reserved for null or missing points.
	 * @since 10.2.1.0
	 */
	public void setCenterInCategory(boolean centerInCategory) {
		setAttr(Attrs.centerInCategory, centerInCategory);
	}

	/**
	 * Returns the border radius of the series.
	 * <p>Default: {@code 0}
	 * @since 10.2.1.0
	 */
	public Number getBorderRadius() {
		return getAttr(Attrs.borderRadius, 0).asNumber();
	}

	/**
	 * Sets the border radius of the series.
	 * <p>Default: {@code 0}
	 * @since 10.2.1.0
	 */
	public void setBorderRadius(Number borderRadius) {
		setAttr(Attrs.borderRadius, borderRadius);
	}
	/**
	 * Returns the color of the border surrounding each slice.
	 * <p>
	 * Default: #FFFFFF
	 * @since 10.2.1.0
	 */
	public Color getBorderColor() {
		return (Color) getAttr(Attrs.borderColor, new Color("#FFFFFF")).asValue();
	}

	/**
	 * Sets the color of the border surrounding each slice.
	 * Default: #FFFFFF
	 * @since 10.2.1.0
	 */
	public void setBorderColor(Color borderColor) {
		setAttr(Attrs.borderColor, borderColor);
	}

	/**
	 * Sets the color of the border surrounding each slice.
	 * Default: #FFFFFF
	 * @since 10.2.1.0
	 */
	public void setBorderColor(String borderColor) {
		setBorderColor(new Color(borderColor));
	}

	/**
	 * Returns a preferable number of bins.
	 * It is a suggestion, so a histogram may have a different number of bins.
	 * By default it is set to the square root of the base series' data length.
	 * Available options are: {@code square-root}, {@code sturges}, {@code rice}.
	 * You can also define a function which takes a baseSeries as a parameter and should return a positive integer.
	 * <p>
	 * Default: "square-root"
	 */
	public Object getBinsNumber() {
		return getAttr(Attrs.binsNumber, "square-root").asValue();
	}

	/**
	 * Sets a preferable number of bins.
	 * It is a suggestion, so a histogram may have a different number of bins.
	 * Available options are: {@code square-root}, {@code sturges}, {@code rice}.
	 */
	public void setBinsNumber(String binsNumber) {
		if (!("square-root".equals(binsNumber) || "sturges".equals(binsNumber) || "rice".equals(binsNumber)))
			throw new IllegalArgumentException("Unsupported binsNumber: [ " + binsNumber + " ]");
		setAttr(Attrs.binsNumber, binsNumber);
	}

	/**
	 * Sets a preferable number of bins.
	 * It is a suggestion, so a histogram may have a different number of bins.
	 */
	public void setBinsNumber(int binsNumber) {
		setAttr(Attrs.binsNumber, binsNumber);
	}

	/**
	 * Sets a preferable number of bins.
	 * It is a suggestion, so a histogram may have a different number of bins.
	 * You can define a function which takes a baseSeries as a parameter and should return a positive integer.
	 */
	public void setBinsNumber(JavaScriptValue binsNumber) {
		setAttr(Attrs.binsNumber, binsNumber);
	}

	/**
	 * Returns the width of each bin. By default the bin's width is calculated
	 * as {@code (max - min) / number of bins}.
	 * This option takes precedence over binsNumber.
	 * <p>
	 * Default: null
	 */
	public Number getBinWidth() {
		return getAttr(Attrs.binWidth, null).asNumber();
	}

	/**
	 * Sets the width of each bin. By default the bin's width is calculated
	 * as {@code (max - min) / number of bins}.
	 * This option takes precedence over binsNumber.
	 */
	public void setBinWidth(Number binWidth) {
		setAttr(Attrs.binWidth, binWidth);
	}

	/**
	 * 3D columns only. The color of the edges. Similar to {@code borderColor},
	 * except it defaults to the same color as the column.
	 * <p>Default: {@code null}
	 * @since 10.2.1.0
	 */
	public Color getEdgeColor() {
		return (Color) getAttr(Attrs.edgeColor, null).asValue();
	}

	/**
	 * 3D columns only. The color of the edges. Similar to {@code borderColor},
	 * except it defaults to the same color as the column.
	 * <p>Default: {@code null}
	 * @since 10.2.1.0
	 */
	public void setEdgeColor(Color color) {
		setAttr(Attrs.edgeColor, color);
	}

	/**
	 * 3D columns only. The color of the edges. Similar to {@code borderColor},
	 * except it defaults to the same color as the column.
	 * <p>Default: {@code null}
	 * @since 10.2.1.0
	 */
	public void setEdgeColor(String color) {
		setColor(new Color(color));
	}

	/**
	 * 3D columns only.
	 * Returns the width of the colored edges.
	 * <p>
	 * Default: 1
	 * @since 10.2.1.0
	 */
	public Number getEdgeWidth() {
		return getAttr(Attrs.edgeWidth, 1).asNumber();
	}

	/**
	 * 3D columns only.
	 * Sets the width of the colored edges.
	 * @since 10.2.1.0
	 */
	public void setEdgeWidth(Number edgeWidth) {
		setAttr(Attrs.edgeWidth, edgeWidth);
	}

	/**
	 * Returns whether to group non-stacked columns or to let them render independent of each other.
	 * Non-grouped columns will be laid out individually and overlap each other.
	 *
	 * <p>Defaults to false.
	 * @since 10.2.1.0
	 */
	public boolean isGrouping() {
		return getAttr(Attrs.grouping, false).asBoolean();
	}
	/**
	 * Sets whether to group non-stacked columns or to let them render
	 * independent of each other. Non-grouped columns will be laid out
	 * individually and overlap each other.
	 * @since 10.2.1.0
	 */
	public void setGrouping(boolean grouping) {
		setAttr(Attrs.grouping, grouping);
	}

	/**
	 * Returns padding between each value groups, in x axis units.
	 * <p>Default: 0
	 * @since 10.2.1.0
	 */
	public Number getGroupPadding() {
		return getAttr(Attrs.groupPadding, 0).asNumber();
	}

	/**
	 * Sets padding between each value groups, in x axis units.
	 * @since 10.2.1.0
	 */
	public void setGroupPadding(Number groupPadding) {
		setAttr(Attrs.groupPadding, groupPadding);
	}

	/**
	 * Returns the spacing between columns on the Z Axis in a 3D chart.
	 * <p>
	 * Default: 1
	 * @since 10.2.1.0
	 */
	public Number getGroupZPadding() {
		return getAttr(Attrs.groupZPadding, 1).asNumber();
	}

	/**
	 * Sets the spacing between columns on the Z Axis in a 3D chart.
	 * <p>
	 * Default: 1
	 * @since 10.2.1.0
	 */
	public void setGroupZPadding(Number groupZPadding) {
		setAttr(Attrs.groupZPadding, groupZPadding);
	}

	/**
	 * Returns the maximum allowed pixel width for a column,
	 * translated to the height of a bar in a bar chart.
	 * This prevents the columns from becoming too wide when there is a small
	 * number of points in the chart.
	 * <p>Default: null
	 * @since 10.2.1.0
	 */
	public Number getMaxPointWidth() {
		return getAttr(Attrs.maxPointWidth, null).asNumber();
	}

	/**
	 * Sets the maximum allowed pixel width for a column,
	 * translated to the height of a bar in a bar chart.
	 * This prevents the columns from becoming too wide when there is a small
	 * number of points in the chart.
	 * @since 10.2.1.0
	 */
	public void setMaxPointWidth(Number maxPointWidth) {
		setAttr(Attrs.maxPointWidth, maxPointWidth);
	}

	/**
	 * Returns the minimal height for a column or width for a bar. By default, 0
	 * values are not shown. To visualize a 0 (or close to zero) point, set the
	 * minimal point length to a pixel value like 3. In stacked column charts,
	 * minPointLength might not be respected for tightly packed values.
	 * <p>Default: 0
	 * @since 10.2.1.0
	 */
	public Number getMinPointLength() {
		return getAttr(Attrs.minPointLength, 0).asNumber();
	}

	/**
	 * Sets the minimal height for a column or width for a bar. By default, 0
	 * values are not shown. To visualize a 0 (or close to zero) point, set the
	 * minimal point length to a pixel value like 3. In stacked column charts,
	 * minPointLength might not be respected for tightly packed values.
	 * @since 10.2.1.0
	 */
	public void setMinPointLength(Number minPointLength) {
		setAttr(Attrs.minPointLength, minPointLength, 0);
	}

	/**
	 * Returns padding between each column or bar, in x axis units.
	 * <p> Default: 0
	 * @since 10.2.1.0
	 */
	public Number getPointPadding() {
		return getAttr(Attrs.pointPadding, 0).asNumber();
	}

	/**
	 * Sets padding between each column or bar, in x axis units. 'pointPadding
	 * padding between each column or bar, in x axis units.
	 * @since 10.2.1.0
	 */
	public void setPointPadding(Number pointPadding) {
		setAttr(Attrs.pointPadding, pointPadding);
	}

	/**
	 * Returns the X axis range that each point is valid for. This determines
	 * the width of the column. On a categorized axis, the range will be 1 by
	 * default (one category unit). On linear and datetime axes, the range will
	 * be computed as the distance between the two closest data points.
	 * <p> Default: null
	 * @since 10.2.1.0
	 */
	public Number getPointRange() {
		return getAttr(Attrs.pointRange, null).asNumber();
	}

	/**
	 * Sets the X axis range that each point is valid for. This determines the
	 * width of the column. On a categorized axis, the range will be 1 by
	 * default (one category unit). On linear and datetime axes, the range will
	 * be computed as the distance between the two closest data points.
	 * @since 10.2.1.0
	 */
	public void setPointRange(Number pointRange) {
		setAttr(Attrs.pointRange, pointRange);
	}

	/**
	 * Returns a pixel value specifying a fixed width for each column or bar.
	 * When <code>null</code>, the width is calculated from the
	 * <code>pointPadding</code> and <code>groupPadding</code>.
	 * <p>Default: null
	 * @since 10.2.1.0
	 */
	public Number getPointWidth() {
		return getAttr(Attrs.pointWidth, null).asNumber();
	}

	/**
	 * Sets a pixel value specifying a fixed width for each column or bar. When
	 * <code>null</code>, the width is calculated from the
	 * <code>pointPadding</code> and <code>groupPadding</code>.
	 * @since 10.2.1.0
	 */
	public void setPointWidth(Number pointWidth) {
		setAttr(Attrs.pointWidth, pointWidth);
	}
}
