/* TreegraphPlotOptions.java

	Purpose:
		
	Description:
		
	History:
		10:47 AM 2023/4/7, Created by jumperchen

Copyright (C) 2023 Potix Corporation. All Rights Reserved.
*/
package org.zkoss.chart.plotOptions;

import java.util.Arrays;
import java.util.List;
import java.util.stream.Collectors;

import org.zkoss.chart.Color;
import org.zkoss.chart.Level;
import org.zkoss.chart.PlotAttribute;
import org.zkoss.chart.TraverseUpButton;
import org.zkoss.chart.util.DynamicalAttribute;
import org.zkoss.lang.Generics;

/**
 * Options for treegraph.
 * @author jumperchen
 * @since 10.3.3.0
 */
public class TreegraphPlotOptions extends SeriesPlotOptions {

	private enum Attrs implements PlotAttribute, DynamicalAttribute {
		allowTraversingTree,
		collapseButton,
		colors,
		levels,
		link,
		traverseUpButton
	}

	/**
	 * When enabled the user can click on a point which is a parent and zoom in
	 * on its children.
	 * <p>
	 * Defaults to false.
	 */
	public boolean isAllowTraversingTree() {
		return getAttr(Attrs.allowTraversingTree, false).asBoolean();
	}
	/**
	 * @param allowTraversingTree
	 * @see #isAllowTraversingTree()
	 */
	public void setAllowTraversingTree(boolean allowTraversingTree) {
		setAttr(Attrs.allowTraversingTree, allowTraversingTree, false);
	}

	/**
	 * Returns the options applied to collapse Button.
	 */
	public CollapseButton getCollapseButton() {
		CollapseButton options = (CollapseButton) getAttr(Attrs.collapseButton);
		 if (options == null) {
			 options = new CollapseButton();
			 setCollapseButton(options);
		 }
		 return options;
	}

	/**
	 * Sets the options applied to collapse Button.
	 */
	public void setCollapseButton(CollapseButton collapseButton) {
		setAttr(Attrs.collapseButton, collapseButton);
	}


	/**
	 * Sets the series specific or series type specific color set to apply
	 * instead of the global colors when {@link #isColorByPoint()} is true.
	 */
	public void setColors(List<Color> colors) {
		setAttr(Attrs.colors, colors);
	}

	/**
	 * Sets the series specific or series type specific color set to apply
	 * instead of the global colors when {@link #isColorByPoint()} is true.
	 */
	public void setColors(String... colors) {
		setColors(Arrays.stream(colors).map(Color::new).collect(Collectors.toList()));
	}

	/**
	 * Sets the series specific or series type specific color set to apply
	 * instead of the global colors when {@link #isColorByPoint()} is true.
	 */
	public void setColors(Color... colors) {
		setColors(Arrays.asList(colors));
	}

	/**
	 * Returns the series specific or series type specific color set to apply
	 * instead of the global colors when {@link #isColorByPoint()} is true.
	 */
	public List<Color> getColors() {
		return Generics.cast(getAttr(Attrs.colors, null).asValue());
	}

	/**
	 * Set options on specific levels. Takes precedence over series options,
	 * but not point options.
	 * <p>
	 * Default: null
	 */
	public List<Level> getLevels() {
		return Generics.cast(getAttr(Attrs.levels, null).asValue());
	}

	/**
	 * @param levels
	 * @see #getLevels()
	 */
	public void setLevels(List<Level> levels) {
		setAttr(Attrs.levels, levels);
	}

	/**
	 * Set options on specific levels. Takes precedence over series options,
	 * but not point options.
	 */
	public void setLevels(Level... levels) {
		setLevels(Arrays.asList(levels));
	}


	/**
	 * Returns the link style options, if any. Otherwise, create a new one.
	 */
	public TreeGraphLink getLink() {
		TreeGraphLink link = (TreeGraphLink) getAttr(Attrs.link);
		if (link == null) {
			link = new TreeGraphLink();
			setLink(link);
		}
		return link;
	}

	/**
	 * Sets the link style options.
	 */
	public void setLink(TreeGraphLink link) {
		setAttr(Attrs.link, link);
	}


	/**
	 * Returns options for the button appearing when traversing down in a treegraph, if any.
	 * Otherwise, create a new one.
	 */
	public TraverseUpButton getTraverseUpButton() {
		TraverseUpButton traverseUpButton = (TraverseUpButton) getAttr(Attrs.traverseUpButton);
		if (traverseUpButton == null) {
			traverseUpButton = new TraverseUpButton();
			setTraverseUpButton(traverseUpButton);
		}
		return traverseUpButton;
	}

	/**
	 * Sets options for the button appearing when traversing down in a treemap.
	 */
	public void setTraverseUpButton(TraverseUpButton traverseUpButton) {
		setAttr(Attrs.traverseUpButton, traverseUpButton);
	}
}